Attribute VB_Name = "modHW2shipHOD"
Option Explicit

'********************************************************************************

Public Type texCo
    u As Single
    v As Single
End Type

Public Type triangle
    vtx(1 To 3) As Integer
End Type

Public Type doubleV3
    x As Double
    y As Double
    z As Double
End Type

Public Type fHdr
    tF As String * 4    ' form type (e.g. 'FORM' or 'NRML')
    lF As Long          '   # bytes covering this form (Reversed Long)
End Type

'********************************************************************************

Public Type warnB
    tID As String * 4   ' type ID = 'WARN'
    lF As Long          '   # bytes covering 'WARN' form
    lN As Long          '   # bytes in owner name
    nWrn As String      ' warning text
End Type

Public Type errrB
    tID As String * 4   ' type ID = 'ERRR'
    lF As Long          '   # bytes covering 'ERRR' form
    lN As Long          '   # bytes in owner name
    nErr As String      ' error text
End Type

Public Type ownrB
    tID As String * 4   ' type ID = 'OWNR'
    lF As Long          '   # bytes covering 'OWNR' form
    lN As Long          '   # bytes in owner name (MINUS ONE !? -> see MsgBox usage in ReadINFO procedure).
    nOw As String       ' owner name
End Type

Public Type infoB
    tID As String * 4   ' type ID = 'INFO'
    ownrFb As ownrB
    warnFb As warnB
    errrFb As errrB
End Type

'***************************************************************************************************
'   --- start SSUB chunk def.

Public Type faceList
    noFcIdcs As Long        ' Number of indices in the faces index list to follow.
    faceIdx() As Integer    ' List of face indices
End Type

Public Type ssubB       ' Surface SUB-division. Contains lists of face-indices (starting at zero) of faces as tagged in
                        '  the corresponding TRIS chunk and in the corresponding LOD0 (!) MULT object.
                        '   Faces listing is probably for DX9 shader feed, like indicated by the bit-flags (see below).
                        '    Faces grouping is per 'convex' vertex (i.e. the vertex 'sticks out of the mesh') or
                        '                           'concave' vertex (i.e. the vertex 'falls back into the mesh').
                        ' This means that faces can be listed more than twice. The idea is that these face-indez lists
                        '  help the shaders with the lighting of the shared edges (that either 'stick' out or 'fall back').
    tID As String * 4   ' type ID = 'SSUB'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no form header used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form.
'***
    flag(1 To 3) As Long   ' bit-flags (they're always powers of two), indicating the layer priorities per mesh.
                           '  -> (For 'diffuse', 'glow' & '...' (?) layers?).
    fcLst() As faceList    ' Listing of face-indices (starting at zero) of faces as outlined in the corresponding TRIS chunk.
End Type

Public Type vnrmB       ' Vertex NoRMals combinations which make up face normals for the faces in the corresponding TRIS chunk.
    tID As String * 4   ' type ID = 'VNRM'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no header is used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    noNrmls As Long     ' number of normals
    nrml() As D3DVECTOR ' Interpreted as vertex normals (normalized, three per face), for calculating the resulting face normal.
    noFcs As Long       ' (number of faces) * 3   -> Should comply w. number of TRIS-faces.
    fc() As triangle    ' Not really a triangle, moreover the indices of three normals to be combined
                        '  into one face normal per TRIS-face.
End Type

Public Type trisB       ' Plain triangles
    tID As String * 4   ' type ID = 'TRIS'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    noVtcs As Long      ' number of vertices
    vtx() As D3DVECTOR  ' this time no vertex normals and texture coordinates attached (unlike BMSH vertices).
    noFcs As Long       ' (number of faces) * 3  (*** No separator used).
    fc() As triangle
End Type

Public Type bsphB       ' Interpreted as collision sphere.
    tID As String * 4   ' type ID = 'BSPH'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    centre As D3DVECTOR ' checked OK in drawing program.
    radius As Single    ' checked OK in drawing program.
End Type

Public Type bboxB       ' Interpreted as collision box.
    tID As String * 4   ' type ID = 'BBOX'
    lF As Long          '   # bytes to follow in this chunk (seen when this chunk is part of a 'COLD' form).
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    leftLowerCorner As D3DVECTOR ' checked OK in drawing program.
    edgeSizes As D3DVECTOR       ' checked OK in drawing program.
End Type

Public Type descB
    tID As String * 4   ' type ID = 'DESC'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no header is used.
                        '    NB: contains '1000' or {00 00 03 E8} in BSRM form. (Reversed Long).
    lN As Long          '   # bytes in mesh name
    nM As String        ' mesh name
    lO As Long          '   # bytes in object name
    nO As String        ' object name
    nonGoblin As Long   ' Goblin indicator.
End Type

'***************************************************************************************************
'   IMPORTANT RULE: IT SEEMS THAT THE NUMBER OF BSRM CHUNKS IS DEFINED BY:
'
'       nr. MULTs + nr. GOBGs = nr. BSRMs
'
'   -> CHECK BUILT IN !
'***************************************************************************************************

Public Type bsrmB
    tID As String * 4   ' type ID = 'BSRM'
    descFh As fHdr      ' with 'NRML'
    descFb As descB     ' occurs only once in BSRM chunk
    bboxFh As fHdr      ' with 'NRML'
    bboxFb As bboxB     ' occurs only once in BSRM chunk
    bsphFh As fHdr      ' with 'NMRL'
    bsphFb As bsphB     ' occurs only once in BSRM chunk
    trisFh As fHdr      ' with 'NMRL'
    trisFb As trisB     ' occurs only once in BSRM chunk
    vnrmFh As fHdr      ' with 'NMRL'
    vnrmFb As vnrmB     ' occurs only once in BSRM chunk
    ssubFh As fHdr      ' with 'NMRL'
    ssubFb As ssubB     ' occurs only once in BSRM chunk
End Type

'***
Public Type coldB       ' ( LOD3? ) Looks like complete meshes at a lower LOD.
    tID As String * 4   ' type ID = 'COLD'      NOTE: COLD chunks don't use an 'altLf' or 'lF' variable!
    lO As Long          '   # bytes in object name to follow.
    nO As String        ' Hierarchy object name.
    bboxFb As bboxB     ' occurs once in COLD chunk, without header.
    bsphFb As bsphB     ' occurs once in COLD chunk, without header.
    trisFb As trisB     ' occurs once in COLD chunk, without header.
End Type

Public Type bndvB       ' Interpreted as 'BouNDary Values' -> LARGER than mesh extremes
                        ' Could be something like: 'camara can't zoom closer than'.
    tID As String * 4   ' type ID = 'BNDV'
    lF As Long          '   # bytes in form
    lN As Long          '   # bytes in name to follow (usually 5).
    nAll As String      ' = 'Whole'
    lO As Long          '   # bytes in object name
    nO As String        ' object name
    noGenPts As Long              ' = always 8 (like: generate EIGHT corners from points below).
    leftLowerCorner As D3DVECTOR  ' checked OK in drawing program.
    sepr1(1 To 15) As Long        ' = always all zero
    rightUpperCorner As D3DVECTOR ' checked OK in drawing program.
    sepr2(1 To 4) As Long         ' = always all zero
End Type

'***************************************************************************************************
'   --- start ANIM chunk def.

Public Type keyfData
    kTime As Double     ' Time values ( x 1/30th again! ).
'*** These appear not to be like their variable name! (see vgr_infiltratorpod.hod).
    kPos As D3DVECTOR
    kRot As D3DVECTOR
End Type

Public Type animB
    tID As String * 4
    noAnimPars As Long  ' number of animation parameters in this form = always 1 ? (Reversed Long).
    lNpar As Long       '   # bytes in animation parameter name.
    nPar As String      ' animation parameter name.
    noKeyfs As Long     ' number of keyframes
    keyframe() As keyfData
    sepr As Double      ' or two Longs? It always seems to be zero.
End Type

'   --- end ANIM chunk def.
'   --- start MRKR chunk def.

Public Type keyfB
    tID As String * 4   ' type ID = 'KEYF'
    ' No more data in KEYF form seen anywhere. Regarded as trailer for the MRKR form.
End Type

Public Type headB       ' Might be something like a 'damage point'.
    tID As String * 4   ' type ID = 'HEAD'
    noHds As Long       ' number of markers in this form (= always 1, Reversed Long).
    lN As Long          '   # bytes in name
    nHe As String       ' marker name
    lNpo As Long        '   # bytes in type name for marker connected object or joint.
    nPo As String       ' Mostly 'Root', also seen 'moshipdoor', so may point to parent hierarchy object.
                        '  With the vgr_planetkiller, all 'Joint$#' are named here.
'***
    sepr As Double         ' = always 0 ?  No, see vgr_infiltratorpod.hod, 'markerB'.
    position As doubleV3    ' found after development of the kpr_sajuuk.hod markers.
    rotation As doubleV3    ' found after development of the kpr_sajuuk.hod markers.
    keyfFh As fHdr          ' with 'FORM'
    keyfFb As keyfB         ' occurs only once per HEAD chunk.
End Type

Public Type mrkrB
    tID As String * 4   ' type ID = 'MRKR'
    headFh As fHdr      ' with 'NRML'
    headFb As headB     ' occurs only once per MRKR chunk.
End Type

'   --- end MRKR chunk def.
'   --- start Dock paths def.

Public Type pathPoint
    ' 64 bytes per point
    pCo As D3DVECTOR    'position?
    pOr As D3DVECTOR    'orientation?
'*** Below: probably spline parameters or flags
    flag0 As Long       ' must be Long!
    flag1 As Single     ' must be Single!     (All investigated)
    flag2 As Long       ' must be Long!
    flag3 As Single     ' must be Single!
    flag4() As Long     ' must be Long!     ' Usually there are TEN flags all together except e.g the meg_progenitorpowermodule.hod (others?).
End Type

Public Type shpPath
    lN As Long          '   # bytes in path name
    nP As String        ' path name
    lO As Long          '   # bytes in hierarchy object name
    nO As String        ' name of the hierarchy object this path is connected to.
                        '  (usually 'world', i.e. path described in coordinates relative to
                        '    whole ship).
    unknown8 As Long    ' = always  6
    noPars As Long      ' Says how many parameters / flags are in the path-points.
    flag1 As Long       ' Flags are either 0 or 1 (Delphy). Used for indicating path priority?
    flag2 As Long       '  They're not excluding each other (see paths of hgn_moship).
    flag3 As Long       ' -> flag3 is probably a reversed Long (see hgn_shipyard.hod).
    lNSfs As Long       '   # bytes in names of ship families to use this path.
    nSfs As String      ' names of ship families to use this path, separated by comma & space.
    flag4 As Long       ' This parameter is either 1 or 0 (Pesmontis).
    lAPs As Long        '   # bytes in names of alternative (?) paths.
    nAPs As String      ' alternative path names, separated by comma & space.
    noPpoints As Long   ' number of points in this path.
    pP() As pathPoint
End Type

Public Type dockB       ' Docking paths don't seem to be necessary for capture points.
    tID As String * 4   ' type ID = 'DOCK'
    lF As Long          '   # bytes in DOCK chunk
    noPaths As Long     ' number of dock paths in DOCK chunk.
    path() As shpPath
End Type

'   --- end Dock paths def.
'   --- start Navlight def.

Public Type shpNavl     ' Data chunk per navlight
    lHo As Long         '   # bytes in navlight name
    nHo As String       ' name of hierarchy object to connect to (i.e. navlight name, because these appear in the hierarchy too).
'***
    n(1 To 4) As Single ' RGBA / ARGB values? -> Delphy: all Single
    lT As Long          '   # bytes in navlight type-name
    tID As String       ' navlight type name
'***
    o(1 To 5) As Single     ' all Single (Delphy). o(5) is not always equal to 'n(2)': see NavLight_01, NavLight_02 & NavLight_03 of hgn_moship.hod.
    sepr(1 To 2) As Byte    ' Each of these bytes can either be {00} or {01}.
End Type

Public Type navlB
    tID As String * 4   ' type ID = 'NAVL'
    altLf As Long       '  (= always 3) (Reversed Long).
                        ' Used alternatively to contain form length (# bytes) if no header is used.
    noNavls As Long     ' number of navlights in this form
    navl() As shpNavl
End Type
    
'   --- end Navlight def.
    
Public Type burnB
    tID As String * 4   ' type ID = 'BURN'
    lF As Long          '   # bytes in BURN chunk (Reversed Long).
    lN As Long          '   # bytes in burn name
    nB As String        ' burn name
    lHo As Long         '   # bytes in hierarchy object name
    nHo As String       ' name of hierarchy object to connect to.
    noConnPnts As Long    ' (= always 5) may be number of connection points.
    unknown4 As Long    ' (= always 1)
    cPnt() As D3DVECTOR ' 'Points-in-a-row': like connection points at different LODs (because of NLIPS trouble?).
End Type

'    --- Start ETSH def.

Public Type etshD
    noConnPnts As Long  ' Number of connection points (can be variable for various ETSH chunks in same HOD).
    cPnt() As D3DVECTOR
End Type

Public Type etshB       ' "Engine Trail SHape" chunk
    tID As String * 4   ' type ID = 'ETSH'
    lF As Long          '   # bytes in ETSH chunk (Reversed Long).
    lN As Long          '   # bytes in shape name
    nS As String        ' Shape name
    lHo As Long         '   # bytes in hierarchy object name
    nHo As String       ' Name of hierarchy object to connect to.
    tSet() As etshD     ' The hgn_pulsarcorvette shows multiple parts within one ETSH chunk.
End Type

'   --- End ETSH def.

'***************************************************************************************************
' BMSH chunk definition (!) - INTERMEZZO -
'  (placed here because used by GLOW, GOBG and MULT chunks).

Public Type vtx1B       ' "1B" relates to the 'MULT' BMSH identifier.
    vCo As D3DVECTOR    ' vertex coordinates
    sepr1 As Long       ' '{00 00 80 3F} (interpreted as a separator)
    vN As D3DVECTOR     ' vertex normal
    sepr2 As Long       ' '{00 00 80 3F} (interpreted as a separator)
    tCo As texCo        ' texture coordinates
End Type

Public Type shpMesh
    texTag As Long      ' pointer to a shader (therefor points to a texture group)
    dwordsPerVtx As Long    '  =  {1B 00 00 00} for ship-mesh BMSH chunks,
                            ' but {0B 00 00 00} for ship-glow BMSH chunks,
                            ' and {05 00 00 00} for background BMSH chunks
                            ' => to be used as additional identifier?
    noVtcs As Long      ' number of vertices in this mesh part.
    shipVtx() As vtx1B
    seprInt As Integer
    seprLong As Long
    noFcs As Long       ' (number of faces) * 3 in this mesh part.
    shipFc() As triangle
End Type

Public Type bmshB
    tID As String * 4   ' type ID = 'BMSH'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no header is used.
    LOD As Long         ' LOD number. Contains zero when used in GLOW chunks.
    noMshs As Long      ' number of meshes in this BMSH chunk.
    shipmesh() As shpMesh
End Type

Public Type bmshC
    bmshFh As fHdr
    bmshFb As bmshB
End Type

'***************************************************************************************************

Public Type glowB       ' Appears directly after ETSH chunk (see sp_tanker.hod).
    tID As String * 4   ' type ID = 'GLOW'
    tFi As String * 4   ' with 'NRML'
    lFi As Long         '  # bytes in INFO chunk.
    tIDi As String * 4  ' type ID = 'INFO'. NB: special type of INFO chunk, not separately defined.
    altLfi As Long      ' *** Used alternatively to contain form length (# bytes) if no header is used.
    lN As Long          '   # bytes in mesh name.
    nGl As String       ' glow-mesh name.
    lHo As Long         '   # bytes in hierarchy object name.
    nHo As String       ' name of hierarchy object to connect to.
    LOD As Long         ' LOD number. Now a BMSH chunk LOD number contains only zero.
    bmshFh As fHdr      ' occurs only ONCE per GLOW chunk.
    bmshFb As bmshB     ' occurs only ONCE per GLOW chunk.
End Type

Public Type hierObj
    lO As Long              '   # bytes in object name.
    nO As String            ' object name
    lP As Long              '   # bytes in parent name ( = 0 for 'Root' object).
    nP As String            ' parent name
    cldPos As D3DVECTOR     ' position relative to parent.
    cldRot As D3DVECTOR     ' orientation relative to parent.
    cldScl As D3DVECTOR     ' possibly scale, but not sure (seen (1,1,1) only).
    cldWrl As D3DVECTOR     ' position relative to root object (according to development of vgr_planetkiller.hod).
    sepr(1 To 3) As Byte    ' {01 01 01} understood as separator.
End Type

Public Type hierB
    tID As String * 4   ' with 'HIER'
    lF As Long          '   # bytes in HIER chunk
    noHobjects As Long  ' number of objects in hierarchy.
    hObj() As hierObj
End Type

'***************************************************************************************************

Public Type glowC
    glowFh As fHdr      ' with 'FORM'
    glowFb As glowB
End Type

Public Type animC       ' with 'FORM'
    animFh As fHdr
    animFb As animB
End Type

Public Type mrkrC
    mrkrFh As fHdr      ' with 'FORM'
    mrkrFb As mrkrB
End Type

Public Type coldC       ' with 'FORM'
    coldFh As fHdr
    coldFb As coldB
End Type

Public Type bsrmC
    bsrmFh As fHdr      ' with 'FORM'
    bsrmFb As bsrmB
End Type

Public Type dtrmB       ' The order in which some of these chunks appear probably follows their naming
                        '  sequence in the HIER chunk.
    tID As String * 4   ' type ID = 'DTRM'
    hierFb As hierB     ' ship type HODs, including asteroids & similar, contain at least one HIER chunk.
                        '   NB: no HIER-header used.
    etshFb() As etshB   ' ETSH chunks can occur multiple times, and do NOT exclude GLOW chunks (see sp_tanker.hod).
                        '   NB: no ETSH header used. Within an ETSH chunk there can be more than one set op points...
    glowCh() As glowC   ' GLOW chunks can occur multiple times, and exclude BURN chunks.
    burnFb() As burnB   ' BURN chunks can occur multiple times, and exclude GLOW chunks.
                        ' (ETSH does NOT always include BURN -> hgn_shipyard, hgn_ecmprobe).
                        '  (# o. BURN chunks does not always match # o. engines in HIER chunk: hgn_interceptor).
                        '   NB: no BURN-header used.
    navlFh As fHdr      ' form type = 'NRML'. NB: NOT a 'FORM' type but an 'NRML' type (!).
                        '  This probably HAS to be, read-algorith-wise!
    navlFb As navlB     ' a NAVL chunk appears only ONCE if at all.
    dockFb As dockB     ' a DOCK chunk appears only ONCE if at all. NB: no DOCK-header used.
    animCh() As animC   ' doesn't occur in EVERY ship type HOD.
    mrkrCh() As mrkrC   ' doesn't occur in EVERY ship type HOD.
    bndvFb As bndvB     ' occurs at least ONCE as if closing off the HIER chunk. No header used.
    coldCh() As coldC   ' can occur multiple times (e.g. hgn_gunturret.hod), but can also never appear (!), independent of how many subobjects!
    bsrmCh() As bsrmC   ' every ship type HOD contains at least one BSRM chunk.
End Type

'***************************************************************************************************

Public Type gobgB       ' "GOBlin Group" chunk.
    tID As String * 4   ' type ID = 'GOBG'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no header is used.
    lN As Long          '   # bytes in goblin group name.
    nG As String        ' goblin name
    lO As Long          '   # bytes in hierarchy object name.
    nO As String        ' hierarchy object name.
    bmshFh As fHdr      ' only ONE BMSH chunk per GOBG chunk.
    bmshFb As bmshB     ' only ONE BMSH chunk per GOBG chunk.
End Type

Public Type multB       ' "MULTi-mesh" chunk
    tID As String * 4   ' type ID = 'MULT'
    altLf As Long       ' Used alternatively to contain form length (# bytes) if no header is used.
    lN As Long          '   # bytes in multi-mesh group name.
    nM As String        ' multi-mesh name
    lO As Long          '   # bytes in hierarchy object name.
    nO As String        ' hierarchy object name (for hierarchy).
    noBMSHs As Long     ' number of BMSH chunks in this MULT chunk.
    bmshCh() As bmshC
End Type

'***************************************************************************************************
'   --- start LMIP chun def.

Public Type lmipBodyHeader
    tID As String * 4   ' type ID = 'DXT5' or '8888' (for TGA), whichever doesn't matter for the read code.
    noMIPls As Long     ' number of MIP MAP levels
    Wi As Long          ' image width
    Hi As Long          ' image height
End Type

Public Type lmipBody
    lmipBh As lmipBodyHeader
    lmipBb() As Byte      ' image contents: can be DDS OR (!) TGA (if TGA, then the texture coordinates are not flipped?).
End Type

Public Type lmipHeader
    tID As String * 4   ' type ID = 'LMIP'
    lF As Long          '   # bytes in complete LMIP chunk, incl. image data chunk.
    lFDN As Long        '   # bytes in {drive, directory & texture file name}.
    nFDN As String      ' drive, directory & texture file name.
End Type

'   --- end LMIP chunk def.
'***************************************************************************************************
'   --- start shader def.

Public Type shpShader
    unknown1 As Long    ' '5 or {05 00 00 00}
    unknown2 As Long    ' '4 or {04 00 00 00}
    texTag  As Long     ' Tag used in the BMSH chunk for texture assignment.
                        '  This number can become -1, e.g. with the hgn_assaultcorvetteelite.hod and the hgn_attackbomberelite.hod.
    lShadN As Long      '   # bytes for the shader name.
                        '  This number can be {FF FF FF FF} (!), e.g. with the hgn_assaultcorvetteelite.hod and the hgn_attackbomberelite.hod.
                        '   With the cap_chunk_01.hod, it's possible to run into 'STAT', i.e. no name-length but a string!
    nShad As String     ' shader name.
End Type

Public Type statB
    tID As String * 4   ' type ID = 'STAT'
    altLf As Long       ' = 1001 (reversed long). Used alternatively to contain form length (# bytes) if no header is used.
    lTgrpN As Long      '   # bytes for the name of the texture group.
    nTgrp As String     ' Texture group name
    lTypeN As Long       '   # bytes for the type name of the texture group
    nType As String     ' Shading types a.o. 'ship' (typically contains alpha for team/stripe),
                        '  'default' (with 'lambert1'-STAT), 'thruster' (typically w. '$diffuseOn' and '$diffuseOff'),
                        '   'badge' (a.o. w. shader '$mask'), 'matte', 'matteAlpha', 'diffuseA' & 'diffuseB' (-> bump map for asteroid!), ...
    noShaders As Long   ' Number of shaders in this texture group.
    shader() As shpShader
End Type

'   --- end shader def.
'***************************************************************************************************

Public Type statE
    tID As String * 4       ' type ID = 'STAT'
    '...
End Type

Public Type statC
    statFh As fHdr          ' type ID = 'NRML'
    statFb As statB
End Type

Public Type lmipC
    lmipFh As lmipHeader    ' type ID = 'LMIP'
    lmipFb As lmipBody
End Type

Public Type multC
    multFh As fHdr          ' type ID = 'NRML'
    multFb As multB
End Type

Public Type gobgC
    gobgFh As fHdr          ' type ID = 'NRML'
    gobgFb As gobgB
End Type

Public Type hvmdB
    tID As String * 4       ' type ID = 'HVMD'
    statEl() As statE
    statCh() As statC       ' Not present in HODs with 'statE' chunks.
    lmipCh() As lmipC       ' No LMIP header is used, because the STATs chunks reveal how many textures are present.
                            '  (i.e. the sum of the number of shaders used per texture group).
                            '   An LMIP chunk does not always have to be present (e.g. the hgn_advancedarray.hod).
    multCh() As multC       ' Ship type HODs contain at least one MULT chunk (except the vgr_c_innate_engine.hod).
    gobgCh() As gobgC       ' Not all ship type HODs contain goblins.
End Type

'***************************************************************************************************

Public Type nameB
    tID As String * 4       ' type ID = 'NAME'
    nN As String            ' 'Homeworld2 Multi Mesh File'
End Type

Public Type versB
    tID As String * 4       ' type ID = 'VERS'
    vNo As Long             ' version number = 131072 ? {00 02 00 00}
End Type

Public Type HODcomp
    versFh As fHdr          ' type ID = 'FORM'
    versFb As versB
    nameFh As fHdr          ' type ID = 'FORM'
    nameFb As nameB
    hvmdFh As fHdr          ' type ID = 'FORM'
    hvmdFb As hvmdB
    dtrmFh As fHdr          ' type ID = 'FORM'
    dtrmFb As dtrmB
    infoFh As fHdr          ' type ID = 'FORM'
    infoFb As infoB
End Type

Public myHOD As HODcomp

    ' Make additional chunk readers for:
    ' - SIMP chunks (a.o. ui HODs);
    ' - LITE chunks (background HODs);
    ' - BGLT chunks (background HODs);
    ' - BGMS (BackGround MeSh);
    ' - BGSG (TGA images in background HODs);
    ' - STRF chunks (background HODs);
    ' - WARN chunks (e.g. hgn_battlecruiserbridge.hod);
    ' - ERRR (e.g. kpr_planetkillerpower.hod);
    ' - GOBS (in fx & backgr HODs);
    ' ...

Public STATpresence As Boolean  ' Algorithm helpers
Public LMIPpresence As Boolean
Public ETSHpresence As Boolean
Public GLOWpresence As Boolean
Public BURNpresence As Boolean
Public NAVLpresence As Boolean
Public COLDpresence As Boolean
Public ANIMpresence As Boolean
Public MRKRpresence As Boolean
Public MRKRseprDiff As String
Public GOBGpresence As Boolean
Public DOCKpresence As Boolean
Public WARNpresence As Boolean
Public ERRRpresence As Boolean

'***************************************************************************************************


Public Sub importHW2shipHOD(shipHODdirANDfileName As String)

Dim myHODno As Integer
Dim tmpStr1 As String * 4
Dim tmpStr2 As String * 4
Dim tmpLong As Long

Dim VERSpassed As Boolean
Dim NAMEpassed As Boolean
Dim HVMDpassed As Boolean
Dim DTRMpassed As Boolean
Dim INFOpassed As Boolean

    VERSpassed = False
    NAMEpassed = False
    HVMDpassed = False
    DTRMpassed = False
    INFOpassed = False
    
    myHODno = FreeFile()
    Open shipHODdirANDfileName For Binary Access Read As myHODno
    
    Get myHODno, , tmpStr1
    
    With myHOD
                
        Do
            
            Select Case tmpStr1
            
            Case "FORM"
                
                Get myHODno, , tmpLong
                Get myHODno, , tmpStr2
                
                Select Case tmpStr2
                
                Case "VERS"
                    
                    If Not VERSpassed Then
                        .versFh.tF = "FORM"
                        .versFh.lF = tmpLong
                        .versFb.tID = "VERS"
                        Get myHODno, , .versFb.vNo
                        Get myHODno, , tmpStr1
                        VERSpassed = True
                    End If
                    
                Case "NAME"
                    
                    If Not NAMEpassed Then
                        .nameFh.tF = "FORM"
                        .nameFh.lF = tmpLong
                        .nameFb.tID = "NAME"
                        GetVarStr myHODno, .nameFb.nN, (ReverseLong(.nameFh.lF) - 4)
                        Get myHODno, , tmpStr1
                        NAMEpassed = True
                    End If
                    
                Case "HVMD"
                    
                    If Not HVMDpassed Then
                        .hvmdFh.tF = "FORM"
                        .hvmdFh.lF = tmpLong
                        .hvmdFb.tID = "HVMD"
                        tmpStr1 = ReadHVMD(myHODno)
                        HVMDpassed = True
                    End If
                    
                Case "DTRM"
                    
                    If Not DTRMpassed Then
                        .dtrmFh.tF = "FORM"
                        .dtrmFh.lF = tmpLong
                        .dtrmFb.tID = "DTRM"
                        tmpStr1 = ReadDTRM(myHODno)
                        DTRMpassed = True
                    End If
                    
                Case Else
                            
                    Exit Do
                            
                End Select

            Case Else
            
                Exit Do
                
            End Select
            
            If VERSpassed And NAMEpassed And HVMDpassed And DTRMpassed And INFOpassed Then Exit Do
            
        Loop
    
    End With
    
    Close myHODno
    
End Sub

Private Function ReadHVMD(hvmdHODno As Integer) As String

Dim i As Integer
Dim tmpStr3 As String * 4
Dim tmpStr4 As String * 4
Dim readStr4 As Boolean
Dim tmpLong As Long      ' Don't merge with other Long definitions!

    STATpresence = False
    LMIPpresence = False
    GOBGpresence = False
    
    readStr4 = True
    
    With myHOD.hvmdFb
        
        Get hvmdHODno, , tmpStr3
        
        Do
            
            Select Case tmpStr3
            
            Case "STAT"
                
                i = 1
                
                Do
                    
                    MsgBox "Not implemented yet."
                    
                    Exit Do
                    
                    Get hvmdHODno, , tmpStr3
                    If tmpStr3 <> "STAT" Then Exit Do
                    
                Loop
                
            Case "NRML"
                
                If readStr4 Then
                    Get hvmdHODno, , tmpLong
                    Get hvmdHODno, , tmpStr4
                End If
                
                Select Case tmpStr4
                    
                Case "STAT"
                
                    STATpresence = True
                    i = 0

                    Do
                        
                        ReDim Preserve .statCh(i)
                        
                        .statCh(i).statFh.tF = "NRML"
                        .statCh(i).statFh.lF = tmpLong
                        .statCh(i).statFb.tID = "STAT"
                        
                        ReadSTAT hvmdHODno, i
                        
                        Get hvmdHODno, , tmpStr3
                        If tmpStr3 <> "NRML" Then Exit Do
                        Get hvmdHODno, , tmpLong
                        Get hvmdHODno, , tmpStr4
                        If tmpStr4 <> "STAT" Then  ' Happens when no LMIP chunk present.
                            readStr4 = False
                            Exit Do
                        End If
                            
                        i = i + 1
                        
                    Loop
                    
                Case "MULT"
                    
                    i = 1
                    
                    Do
                        
                        ReDim Preserve .multCh(i)
                        
                        .multCh(i).multFh.tF = "NRML"
                        .multCh(i).multFh.lF = tmpLong
                        .multCh(i).multFb.tID = "MULT"
                            
                        ReadMULT hvmdHODno, i
                        
                        Get hvmdHODno, , tmpStr3
                        If tmpStr3 <> "NRML" Then Exit Do  ' When no GOBG chunk follows.
                        Get hvmdHODno, , tmpLong
                        Get hvmdHODno, , tmpStr4
                        If tmpStr4 <> "MULT" Then  ' Happens when a GOBG chunk follows.
                            readStr4 = False
                            Exit Do
                        End If
                        
                        i = i + 1
                        
                    Loop
                   
                Case "GOBG"
                        
                    GOBGpresence = True
                    i = 1
                        
                    Do
                
                        ReDim Preserve .gobgCh(1 To i)
    
                        .gobgCh(i).gobgFh.tF = "NRML"
                        .gobgCh(i).gobgFh.lF = tmpLong
                        .gobgCh(i).gobgFb.tID = "GOBG"
                            
                        ReadGOBG hvmdHODno, i
                        
                        Get hvmdHODno, , tmpStr3
                        If tmpStr3 <> "NRML" Then Exit Do  ' Note the position in the algoritm!
                        Get hvmdHODno, , tmpLong
                        Get hvmdHODno, , tmpStr4
                
                        i = i + 1
                    
                    Loop
                        
                Case Else
                        
                    Exit Do
                        
                End Select
                
            Case "LMIP"
                
                LMIPpresence = True
                i = 0         ' Starting from zero to follow the 'texTag' indices.
                
                Do
                
                    ReDim Preserve .lmipCh(i)
                    
                    .lmipCh(i).lmipFh.tID = "LMIP"
                    Get hvmdHODno, , .lmipCh(i).lmipFh.lF
                    Get hvmdHODno, , .lmipCh(i).lmipFh.lFDN
                    GetVarStr hvmdHODno, .lmipCh(i).lmipFh.nFDN, .lmipCh(i).lmipFh.lFDN
                    Get hvmdHODno, , .lmipCh(i).lmipFb.lmipBh
                    ReDim .lmipCh(i).lmipFb.lmipBb(1 To (ReverseLong(.lmipCh(i).lmipFh.lF) - .lmipCh(i).lmipFh.lFDN - 20))
                    Get hvmdHODno, , .lmipCh(i).lmipFb.lmipBb
                    
                    i = i + 1
                    
                    Get hvmdHODno, , tmpStr3
                    If tmpStr3 <> "LMIP" Then Exit Do
                    
                Loop
                    
            Case Else
                
                Exit Do
            
            End Select
            
        Loop
            
    End With
    
    ReadHVMD = tmpStr3

End Function

Private Function ReadDTRM(dtrmHODno As Integer) As String

Dim i As Integer
Dim tmpStr5 As String * 4
Dim tmpStr6 As String * 4
Dim readStr6 As Boolean
Dim tmpLong As Long      ' Don't merge with other Long definitions!
    
    ETSHpresence = False
    GLOWpresence = False
    BURNpresence = False
    NAVLpresence = False
    COLDpresence = False
    ANIMpresence = False
    MRKRpresence = False
    DOCKpresence = False
    
    With myHOD.dtrmFb
        
        readStr6 = True
        Get dtrmHODno, , tmpStr5
        
        Do
            
            Select Case tmpStr5
            
            Case "HIER"
                
                ReadHIER dtrmHODno
                Get dtrmHODno, , tmpStr5
                
            Case "ETSH"
                
                ETSHpresence = True
                i = 1
                
                Do
                    
                    ReadETSH dtrmHODno, i
                    
                    Get dtrmHODno, , tmpStr5
                    If tmpStr5 <> "ETSH" Then
                        If tmpStr5 = "NRML" Then
                            readStr6 = True
                            Exit Do
                        ElseIf tmpStr5 = "FORM" Then
                            readStr6 = True
                            Exit Do
                        Else
                            readStr6 = False
                            Exit Do
                        End If
                    End If
                    
                    i = i + 1
                    
                Loop
                
            Case "FORM" ' ... then either GLOW, MRKR, COLD, BSRM or INFO chunk.
                
                If readStr6 Then
                    Get dtrmHODno, , tmpLong
                    Get dtrmHODno, , tmpStr6
                End If
                
                Select Case tmpStr6
                
                Case "GLOW"
                    
                    GLOWpresence = True
                    i = 1
                    
                    Do
                        
                        ReDim Preserve myHOD.dtrmFb.glowCh(1 To i)
                        
                        .glowCh(i).glowFh.tF = "FORM"
                        .glowCh(i).glowFh.lF = tmpLong
                        .glowCh(i).glowFb.tID = "GLOW"
                        
                        ReadGLOW dtrmHODno, i
                        
                        Get dtrmHODno, , tmpStr5
                        If tmpStr5 <> "FORM" Then
                            If tmpStr5 = "NRML" Then
                                readStr6 = True
                                Exit Do
                            Else
                                readStr6 = False
                                Exit Do
                            End If
                        End If
                        Get dtrmHODno, , tmpLong
                        Get dtrmHODno, , tmpStr6
                        If tmpStr6 <> "GLOW" Then
                            readStr6 = False
                            Exit Do
                        End If
                        
                        i = i + 1
                        
                    Loop
                    
                Case "MRKR"
                    
                    If MRKRpresence Then            ' Specially for the vgr_infiltratorpod.hod,
                        i = UBound(.mrkrCh)         '  where ANIM chunks occur AMONG MRKR chunks!
                    Else
                        i = 1
                    End If
                    
                    MRKRpresence = True
                    
                    Do
                        
                        ReDim Preserve .mrkrCh(1 To i)
                        
                        .mrkrCh(i).mrkrFh.tF = "FORM"
                        .mrkrCh(i).mrkrFh.lF = tmpLong
                        .mrkrCh(i).mrkrFb.tID = "MRKR"
                        Get dtrmHODno, , .mrkrCh(i).mrkrFb.headFh
                        
                        ReadMRKR dtrmHODno, i
                        
                        Get dtrmHODno, , tmpStr5
                        If tmpStr5 <> "FORM" Then
                            If tmpStr5 = "NRML" Then
                                readStr6 = True
                                Exit Do
                            Else
                                readStr6 = False
                                Exit Do
                            End If
                        End If
                        Get dtrmHODno, , tmpLong
                        Get dtrmHODno, , tmpStr6
                        If tmpStr6 <> "MRKR" Then
                            readStr6 = False
                            Exit Do
                        End If
                        
                        i = i + 1
                        
                    Loop
                    
                Case "COLD"
                    
                    COLDpresence = True
                    i = 1
                    
                    Do
                        
                        ReadCOLD dtrmHODno, i, tmpLong
                        
                        Get dtrmHODno, , tmpStr5
                        If tmpStr5 <> "FORM" Then
                            If tmpStr5 = "NRML" Then
                                readStr6 = True
                                Exit Do
                            Else
                                readStr6 = False
                                Exit Do
                            End If
                        End If
                        Get dtrmHODno, , tmpLong
                        Get dtrmHODno, , tmpStr6
                        If tmpStr6 <> "COLD" Then
                            readStr6 = False
                            Exit Do
                        End If
                        
                        i = i + 1
                        
                    Loop
                    
                Case "BSRM"
                    
                    i = 1
                    
                    Do
                        
                        ReadBSRM dtrmHODno, i, tmpLong
                        
                        Get dtrmHODno, , tmpStr5
                        Get dtrmHODno, , tmpLong
                        Get dtrmHODno, , tmpStr6            ' tmpStr6 can now be read without restraint, because
                        If tmpStr6 <> "BSRM" Then           '  now only BSRM or INFO chunks follow.
                            readStr6 = False
                            Exit Do
                        End If
                        
                        i = i + 1
                        
                    Loop
                    
                Case "INFO"
                    
                    myHOD.infoFh.tF = "FORM"
                    myHOD.infoFh.lF = tmpLong
                    myHOD.infoFb.tID = "INFO"
                    tmpStr5 = ReadINFO(dtrmHODno)           ' Call function for reading INFO chunks.
                    
                Case Else
                    
                    Exit Do
                    
                End Select
                
            Case "BURN"
                
                BURNpresence = True
                i = 1
                
                Do
                    
                    ReadBURN dtrmHODno, i
                    
                    Get dtrmHODno, , tmpStr5
                    If tmpStr5 <> "BURN" Then
                        If tmpStr5 = "NRML" Then
                            readStr6 = True
                            Exit Do
                        ElseIf tmpStr5 = "FORM" Then
                            readStr6 = True
                            Exit Do
                        Else
                            readStr6 = False
                            Exit Do
                        End If
                    End If
                    
                    i = i + 1
                    
                Loop
                
            Case "NRML" ' ... then either NAVL or ANIM chunks. Others?
                
                If readStr6 Then
                    Get dtrmHODno, , tmpLong
                    Get dtrmHODno, , tmpStr6
                End If
                
                Select Case tmpStr6
                
                Case "NAVL"
                    
                    NAVLpresence = True
                    
                    ReadNAVL dtrmHODno, tmpLong
                    
                    Get dtrmHODno, , tmpStr5
                    If tmpStr5 <> "NRML" Then
                        If tmpStr5 = "FORM" Then
                            readStr6 = True
                        Else
                            readStr6 = False
                        End If
                    Else
                        readStr6 = True
                    End If
                    
                Case "ANIM"
                    
                    ANIMpresence = True
                    i = 1
                    
                    Do
                        
                        ReadANIM dtrmHODno, i, tmpLong
                        
                        Get dtrmHODno, , tmpStr5
                        If tmpStr5 <> "NRML" Then
                            If tmpStr5 = "FORM" Then
                                readStr6 = True
                                Exit Do
                            Else
                                readStr6 = False
                                Exit Do
                            End If
                        End If
                        Get dtrmHODno, , tmpLong
                        Get dtrmHODno, , tmpStr6
                        If tmpStr6 <> "ANIM" Then
                            readStr6 = False
                            Exit Do
                        End If
                                
                        i = i + 1
                        
                    Loop
                            
                Case Else
                            
                    Exit Do
                            
                End Select
                        
            Case "DOCK"
                        
                DOCKpresence = True
                
                ReadDOCK dtrmHODno
                
                Get dtrmHODno, , tmpStr5
                If tmpStr5 = "NRML" Then
                    readStr6 = True
                ElseIf tmpStr5 = "FORM" Then
                    readStr6 = True
                Else
                    readStr6 = False
                End If
                        
            Case "BNDV"
                        
                ReadBNDV dtrmHODno
                
                Get dtrmHODno, , tmpStr5
                If tmpStr5 = "NRML" Then
                    readStr6 = True
                ElseIf tmpStr5 = "FORM" Then
                    readStr6 = True
                Else
                    readStr6 = False
                End If
                
            Case Else
                        
                Exit Do
                        
            End Select
                   
        Loop
                
    End With
    
    ReadDTRM = tmpStr5

End Function

Private Function ReadINFO(infoHODno As Integer) As String

Dim tmpStr7 As String * 4
Dim tmpStr8 As String
    
    WARNpresence = False
    ERRRpresence = False
    
    With myHOD.infoFb
    
        Do
            If Not EOF(infoHODno) Then
                Get infoHODno, , tmpStr7
            Else
                ReadINFO = ""
                Exit Do
            End If
    
            Select Case tmpStr7
        
            Case "OWNR"
                
                .ownrFb.tID = "OWNR"
                Get infoHODno, , .ownrFb.lF
                Get infoHODno, , .ownrFb.lN
                GetVarStr infoHODno, .ownrFb.nOw, (.ownrFb.lN - 1)
                MsgBox "Brought to you by designer: " & Left(UCase(Left(.ownrFb.nOw, 2)), 1) & ". " & Right(UCase(Left(.ownrFb.nOw, 2)), 1) _
                                                      & Right(.ownrFb.nOw, (.ownrFb.lN - 3)) & vbCrLf & Space(6) & " ( Relic Entertainment )" _
                                                      & vbCrLf & vbCrLf & vbCrLf & Space(12) & "( Extracted by Pesmontis, 2003 )."
            Case "WARN"
                WARNpresence = True
                .warnFb.tID = "WARN"
                Get infoHODno, , .warnFb.lF
                Get infoHODno, , .warnFb.lN
                GetVarStr infoHODno, .warnFb.nWrn, .warnFb.lN
                MsgBox "This HOD contains a warning: '" & .warnFb.nWrn
            
            Case "ERRR"
            
                ERRRpresence = True
                .errrFb.tID = "ERRR"
                Get infoHODno, , .errrFb.lF
                Get infoHODno, , .errrFb.lN
                GetVarStr infoHODno, .errrFb.nErr, .errrFb.lN
                MsgBox "Error message in this HOD says: '" & .errrFb.nErr
            
            Case Else
        
                Exit Do
            
            End Select
            
        Loop
        
        ReadINFO = ""
        
    End With

End Function

Private Sub ReadEliteSTAT(estatHODno As Integer, noEStat As Integer)

    With myHOD.hvmdFb.statEl(noEStat)
    
        ' ...
    
    End With

End Sub

Private Sub ReadSTAT(statHODno As Integer, noSTAT As Integer)

Dim j As Integer

    With myHOD.hvmdFb.statCh(noSTAT).statFb
        
        Get statHODno, , .altLf
        Get statHODno, , .lTgrpN
        GetVarStr statHODno, .nTgrp, .lTgrpN
        Get statHODno, , .lTypeN
        GetVarStr statHODno, .nType, .lTypeN
        Get statHODno, , .noShaders
                
        For j = 1 To .noShaders
            ReDim Preserve .shader(1 To j) ' Do not define before the 'For...to' loop -> will go wrong.
            Get statHODno, , .shader(j).unknown1
            Get statHODno, , .shader(j).unknown2
            Get statHODno, , .shader(j).texTag
            Get statHODno, , .shader(j).lShadN
            GetVarStr statHODno, .shader(j).nShad, .shader(j).lShadN
        Next j
                
    End With

End Sub

Private Sub ReadMULT(multHODno As Integer, noMULT As Integer)

Dim j As Integer
'Dim k As Integer

    With myHOD.hvmdFb.multCh(noMULT).multFb
    
        Get multHODno, , .altLf
        Get multHODno, , .lN
        GetVarStr multHODno, .nM, .lN
        Get multHODno, , .lO
        GetVarStr multHODno, .nO, .lO
        Get multHODno, , .noBMSHs
        
        ReDim .bmshCh(1 To .noBMSHs)
                
        For j = 1 To .noBMSHs
        
            Get multHODno, , .bmshCh(j).bmshFh
            
            ReadBMSH multHODno, .bmshCh(j).bmshFb
            
            'Get multHODno, , .bmshCh(j).bmshFb.tID
            'Get multHODno, , .bmshCh(j).bmshFb.altLf
            'Get multHODno, , .bmshCh(j).bmshFb.LOD
            'Get multHODno, , .bmshCh(j).bmshFb.noMshs
            'ReDim .bmshCh(j).bmshFb.shipmesh(1 To .bmshCh(j).bmshFb.noMshs)
            'For k = 1 To .bmshCh(j).bmshFb.noMshs
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).texTag
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).dwordsPerVtx
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).noVtcs
            '    ReDim .bmshCh(j).bmshFb.shipmesh(k).shipVtx(1 To .bmshCh(j).bmshFb.shipmesh(k).noVtcs)
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).shipVtx
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).seprInt
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).seprLong
                        
            '    MsgBox "MULT: The separator values between vertices & faces are ( seprInt = " _
            '                    & .bmshCh(j).bmshFb.shipmesh(k).seprInt _
            '                    & ", seprLong = " & .bmshCh(j).bmshFb.shipmesh(k).seprLong & " )." & vbCrLf _
            '                    & "The 'dwordsPerVtx' parameter is: " & .bmshCh(j).bmshFb.shipmesh(k).dwordsPerVtx
                        
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).noFcs
            '    .bmshCh(j).bmshFb.shipmesh(k).noFcs = .bmshCh(j).bmshFb.shipmesh(k).noFcs / 3
            '    ReDim .bmshCh(j).bmshFb.shipmesh(k).shipFc(1 To .bmshCh(j).bmshFb.shipmesh(k).noFcs)
            '    Get multHODno, , .bmshCh(j).bmshFb.shipmesh(k).shipFc
            'Next k
            
        Next j
    
    End With

End Sub

Private Sub ReadGOBG(gobgHODno As Integer, noGobg As Integer)

'Dim j As Integer

    With myHOD.hvmdFb.gobgCh(noGobg).gobgFb
    
        Get gobgHODno, , .altLf
        Get gobgHODno, , .lN
        GetVarStr gobgHODno, .nG, .lN
        Get gobgHODno, , .lO
        GetVarStr gobgHODno, .nO, .lO
        Get gobgHODno, , .bmshFh
        
        ReadBMSH gobgHODno, .bmshFb
        
        'Get gobgHODno, , .bmshFb.tID
        'Get gobgHODno, , .bmshFb.altLf
        'Get gobgHODno, , .bmshFb.LOD
        'Get gobgHODno, , .bmshFb.noMshs      ' ... define types '.bmshFb.bmshBh' and '.bmshFb.bmshBb'
                
        'ReDim .bmshFb.shipmesh(1 To .bmshFb.noMshs)
        'For j = 1 To .bmshFb.noMshs
        '    Get gobgHODno, , .bmshFb.shipmesh(j).texTag
        '    Get gobgHODno, , .bmshFb.shipmesh(j).dwordsPerVtx
        '    Get gobgHODno, , .bmshFb.shipmesh(j).noVtcs
        '    ReDim .bmshFb.shipmesh(j).shipVtx(1 To .bmshFb.shipmesh(j).noVtcs)
        '    Get gobgHODno, , .bmshFb.shipmesh(j).shipVtx
        '    Get gobgHODno, , .bmshFb.shipmesh(j).seprInt
        '    Get gobgHODno, , .bmshFb.shipmesh(j).seprLong
        '    Get gobgHODno, , .bmshFb.shipmesh(j).noFcs
        '    .bmshFb.shipmesh(j).noFcs = .bmshFb.shipmesh(j).noFcs / 3
        '    ReDim .bmshFb.shipmesh(j).shipFc(1 To .bmshFb.shipmesh(j).noFcs)
        '    Get gobgHODno, , .bmshFb.shipmesh(j).shipFc
        'Next j
                
    End With

End Sub

Private Sub ReadGLOW(glowHODno As Integer, noGlow As Integer)

'Dim j As Integer

    With myHOD.dtrmFb.glowCh(noGlow).glowFb
        
        Get glowHODno, , .tFi
        Get glowHODno, , .lFi
        Get glowHODno, , .tIDi
        Get glowHODno, , .altLfi
        Get glowHODno, , .lN
        GetVarStr glowHODno, .nGl, .lN
        Get glowHODno, , .lHo
        GetVarStr glowHODno, .nHo, .lHo
        Get glowHODno, , .LOD
        Get glowHODno, , .bmshFh
        
        ReadBMSH glowHODno, .bmshFb
        
        'Get glowHODno, , .bmshFb.tID
        'Get glowHODno, , .bmshFb.altLf
        'Get glowHODno, , .bmshFb.LOD
        'Get glowHODno, , .bmshFb.noMshs
        'ReDim .bmshFb.shipmesh(1 To .bmshFb.noMshs)
        'For j = 1 To .bmshFb.noMshs
        '    Get glowHODno, , .bmshFb.shipmesh(j).texTag
        '    Get glowHODno, , .bmshFb.shipmesh(j).dwordsPerVtx
        '    Get glowHODno, , .bmshFb.shipmesh(j).noVtcs
        '    ReDim .bmshFb.shipmesh(j).shipVtx(1 To .bmshFb.shipmesh(j).noVtcs)
        '    Get glowHODno, , .bmshFb.shipmesh(j).shipVtx
        '    Get glowHODno, , .bmshFb.shipmesh(j).seprInt
        '    Get glowHODno, , .bmshFb.shipmesh(j).seprLong
            
        '    MsgBox "GLOW: The separator values between vertices & faces are ( seprInt = " _
        '            & .bmshFb.shipmesh(j).seprInt _
        '            & ", seprLong = " & .bmshFb.shipmesh(j).seprLong & " )." & vbCrLf _
        '            & "'dwordsPerVertex: " & .bmshFb.shipmesh(j).dwordsPerVtx
            
        '    Get glowHODno, , .bmshFb.shipmesh(j).noFcs
        '    .bmshFb.shipmesh(j).noFcs = .bmshFb.shipmesh(j).noFcs / 3
        '    ReDim .bmshFb.shipmesh(j).shipFc(1 To .bmshFb.shipmesh(j).noFcs)
        '    Get glowHODno, , .bmshFb.shipmesh(j).shipFc
        'Next j
        
    End With


End Sub

Private Sub ReadBMSH(bmshHODno As Integer, bmshBody As bmshB)

Dim j As Long

    With bmshBody

        Get bmshHODno, , .tID
        Get bmshHODno, , .altLf
        Get bmshHODno, , .LOD
        Get bmshHODno, , .noMshs
        ReDim .shipmesh(1 To .noMshs)
        For j = 1 To .noMshs
            Get bmshHODno, , .shipmesh(j).texTag
            Get bmshHODno, , .shipmesh(j).dwordsPerVtx
            Get bmshHODno, , .shipmesh(j).noVtcs
            ReDim .shipmesh(j).shipVtx(1 To .shipmesh(j).noVtcs)
            Get bmshHODno, , .shipmesh(j).shipVtx
            Get bmshHODno, , .shipmesh(j).seprInt
            Get bmshHODno, , .shipmesh(j).seprLong
            Get bmshHODno, , .shipmesh(j).noFcs
            .shipmesh(j).noFcs = .shipmesh(j).noFcs / 3
            ReDim .shipmesh(j).shipFc(1 To .shipmesh(j).noFcs)
            Get bmshHODno, , .shipmesh(j).shipFc
        Next j
        
    End With

End Sub

Private Sub ReadHIER(hierHODno As Integer)

Dim i As Integer

    With myHOD.dtrmFb.hierFb
    
        .tID = "HIER"
        Get hierHODno, , .lF
        Get hierHODno, , .noHobjects
        ReDim .hObj(.noHobjects)
        For i = 1 To .noHobjects
            Get hierHODno, , .hObj(i).lO
            GetVarStr hierHODno, .hObj(i).nO, .hObj(i).lO
            Get hierHODno, , .hObj(i).lP
            GetVarStr hierHODno, .hObj(i).nP, .hObj(i).lP   ' It doesn't matter whether the length of the parent name is zero.
            Get hierHODno, , .hObj(i).cldPos
            Get hierHODno, , .hObj(i).cldRot
            Get hierHODno, , .hObj(i).cldScl
            Get hierHODno, , .hObj(i).cldWrl
            Get hierHODno, , .hObj(i).sepr                  ' Values are always zero or one.
        Next i
        
    End With

End Sub

Private Sub ReadETSH(etshHODno As Integer, noEtsh As Integer)

Dim j As Integer
Dim k As Integer
Dim tmpLong As Long
'Dim etshFileNo As Integer

    ReDim Preserve myHOD.dtrmFb.etshFb(1 To noEtsh)

    With myHOD.dtrmFb.etshFb(noEtsh)
    
        .tID = "ETSH"
        Get etshHODno, , .lF
        tmpLong = ReverseLong(.lF)  ' safety counter ***
        Get etshHODno, , .lN
        GetVarStr etshHODno, .nS, .lN
        Get etshHODno, , .lHo
        GetVarStr etshHODno, .nHo, .lHo
        tmpLong = tmpLong - (8 + .lN + .lHo)  ' (FOUR bytes for BOTH name-length parameters, i.e. 2 x 4 = 8).
        k = 0
        While tmpLong > 0
            k = k + 1
            ReDim Preserve .tSet(1 To k)
            Get etshHODno, , .tSet(k).noConnPnts
            ReDim .tSet(k).cPnt(1 To .tSet(k).noConnPnts)
            Get etshHODno, , .tSet(k).cPnt
            tmpLong = tmpLong - (4 + .tSet(k).noConnPnts * 12)  ' (TWELVE bytes per connection point, FOUR bytes for the number of connection points).
        Wend
                
        ' Output in order to check points in drawing program.
        'etshFileNo = FreeFile
        'Open "etshCpoints.obj" For Append As #etshFileNo
        'Print #etshFileNo, ""
        'Print #etshFileNo, "g " & .nS
        'For k = 1 To UBound(.tSet)
        '    For j = 1 To .tSet(k).noConnPnts
        '        Print #etshFileNo, "v " & .tSet(k).cPnt(j).x & " " & .tSet(k).cPnt(j).y & " " & .tSet(k).cPnt(j).z
        '    Next j
        '    Print #etshFileNo, "# " & .tSet(k).noConnPnts & " vertices"
        'Next k
        'Print #etshFileNo, ""
        'Close etshFileNo
        ' End of output section.
        
    End With
    
End Sub

Private Sub ReadNAVL(navlHODno As Integer, lFg As Long)
    
Dim j As Integer
    
    With myHOD.dtrmFb
        .navlFh.tF = "NRML"
        .navlFh.lF = lFg
        .navlFb.tID = "NAVL"
        Get navlHODno, , .navlFb.altLf
        Get navlHODno, , .navlFb.noNavls
        ReDim .navlFb.navl(1 To .navlFb.noNavls)
        For j = 1 To .navlFb.noNavls
            Get navlHODno, , .navlFb.navl(j).lHo
            GetVarStr navlHODno, .navlFb.navl(j).nHo, .navlFb.navl(j).lHo
            Get navlHODno, , .navlFb.navl(j).n
            Get navlHODno, , .navlFb.navl(j).lT
            GetVarStr navlHODno, .navlFb.navl(j).tID, .navlFb.navl(j).lT
            Get navlHODno, , .navlFb.navl(j).o
            Get navlHODno, , .navlFb.navl(j).sepr
        Next j
    End With
    
End Sub

Private Sub ReadDOCK(dockHODno As Integer)

Dim j As Integer
Dim k As Integer
'Dim l As Integer
'Dim pathFileNo As Integer

    With myHOD.dtrmFb.dockFb
        
        .tID = "DOCK"
        Get dockHODno, , .lF
        Get dockHODno, , .noPaths
        ReDim .path(1 To .noPaths)
        For j = 1 To .noPaths
            Get dockHODno, , .path(j).lN
            GetVarStr dockHODno, .path(j).nP, .path(j).lN
            Get dockHODno, , .path(j).lO
            GetVarStr dockHODno, .path(j).nO, .path(j).lO
            Get dockHODno, , .path(j).unknown8
            Get dockHODno, , .path(j).noPars
            Get dockHODno, , .path(j).flag1
            Get dockHODno, , .path(j).flag2
            Get dockHODno, , .path(j).flag3
            Get dockHODno, , .path(j).lNSfs
            GetVarStr dockHODno, .path(j).nSfs, .path(j).lNSfs
            Get dockHODno, , .path(j).flag4
            Get dockHODno, , .path(j).lAPs
            GetVarStr dockHODno, .path(j).nAPs, .path(j).lAPs
            Get dockHODno, , .path(j).noPpoints
            ReDim .path(j).pP(1 To .path(j).noPpoints)
            For k = 1 To .path(j).noPpoints
                Get dockHODno, , .path(j).pP(k).pCo
                Get dockHODno, , .path(j).pP(k).pOr
                Get dockHODno, , .path(j).pP(k).flag0
                Get dockHODno, , .path(j).pP(k).flag1
                Get dockHODno, , .path(j).pP(k).flag2
                Get dockHODno, , .path(j).pP(k).flag3
                ReDim .path(j).pP(k).flag4(1 To (.path(j).noPars - 4))
                Get dockHODno, , .path(j).pP(k).flag4
            Next k
        Next j
        
        '  Output in order to check points in drawing program.
        'pathFileNo = FreeFile
        'For j = 1 To .noPaths
        '    Open "path" & j & ".obj" For Append As #pathFileNo
        '    Print #pathFileNo, ""
        '    Print #pathFileNo, "# path '" & .path(j).nP & "' connected to " & .path(j).nO
        '    Print #pathFileNo, "g " & .path(j).nP
        '    For k = 1 To .path(j).noPpoints
        '        Print #pathFileNo, "v " & .path(j).pP(k).pCo.x & " " & .path(j).pP(k).pCo.y & " " & .path(j).pP(k).pCo.z
        '        Print #pathFileNo, "# " & .path(j).pP(k).pOr.x & " " & .path(j).pP(k).pOr.y & " " & .path(j).pP(k).pOr.z
        '        Print #pathFileNo, "# flag0    of this point = " & .path(j).pP(k).flag0
        '        Print #pathFileNo, "# flag1    of this point = " & .path(j).pP(k).flag1
        '        Print #pathFileNo, "# flag2    of this point = " & .path(j).pP(k).flag2
        '        Print #pathFileNo, "# flag3    of this point = " & .path(j).pP(k).flag3
        '        For l = 1 To (.path(j).noPars - 4)
        '            Print #pathFileNo, "# flag4(" & l & ") of this point = " & .path(j).pP(k).flag4(l)
        '        Next l
        '    Next k
        '    Print #pathFileNo, "# " & .path(j).noPpoints & " vertices"
        '    For k = 1 To .path(j).noPpoints
        '        If k < .path(j).noPpoints Then
        '            Print #pathFileNo, "f  " & k & " " & (k + 1) & " " & (k + 1)
        '        End If
        '    Next k
        '    Print #pathFileNo, "# " & (.path(j).noPpoints - 1) & " faces"
        '    Print #pathFileNo, ""
        '    Close #pathFileNo
        'Next j
        ' End of output section.
        
    End With

End Sub

Private Sub ReadMRKR(mrkrHODno As Integer, noMrkr As Integer)



'Dim mrkrFileNo As Integer

    With myHOD.dtrmFb.mrkrCh(noMrkr).mrkrFb.headFb
        
        Get mrkrHODno, , .tID
        Get mrkrHODno, , .noHds   ' This parameter is ALWAYS equal to reversed(one). (Like "Number of markers in this HEAD chunk").
        Get mrkrHODno, , .lN
        GetVarStr mrkrHODno, .nHe, .lN
        Get mrkrHODno, , .lNpo
        GetVarStr mrkrHODno, .nPo, .lNpo
        Get mrkrHODno, , .sepr
        If .sepr <> 0 Then MRKRseprDiff = "Separator with marker '" & .nHe & "' ( " & noMrkr & "th ) is unequal to zero: " & CSng(.sepr)
        Get mrkrHODno, , .position
        Get mrkrHODno, , .rotation
        Get mrkrHODno, , .keyfFh
        Get mrkrHODno, , .keyfFb.tID
        
        ' Output in order to check points in drawing program.
        'mrkrFileNo = FreeFile
        'Open "allMarkers.obj" For Append As #mrkrFileNo
        'Print #mrkrFileNo, "g " & .mrkrFb.headFb.nHe
        'Print #mrkrFileNo, "v " & CSng(.mrkrFb.headFb.position.x) & " " & CSng(.mrkrFb.headFb.position.y) & " " & CSng(.mrkrFb.headFb.position.z)
        'Close #mrkrFileNo
        ' End of output section.
                
    End With
    
End Sub

Private Sub ReadCOLD(coldHODno As Integer, noCold As Integer, lFc As Long)

Dim trisFileNo As Integer
Dim j As Long

    ReDim Preserve myHOD.dtrmFb.coldCh(1 To noCold)

    With myHOD.dtrmFb.coldCh(noCold).coldFh
        .tF = "FORM"
        .lF = lFc
    End With
    
    With myHOD.dtrmFb.coldCh(noCold).coldFb
        .tID = "COLD"
        Get coldHODno, , .lO
        GetVarStr coldHODno, .nO, .lO
        Get coldHODno, , .bboxFb
        Get coldHODno, , .bsphFb
        Get coldHODno, , .trisFb.tID
        Get coldHODno, , .trisFb.lF
        Get coldHODno, , .trisFb.noVtcs
        ReDim .trisFb.vtx(1 To .trisFb.noVtcs)
        Get coldHODno, , .trisFb.vtx
        Get coldHODno, , .trisFb.noFcs
        .trisFb.noFcs = .trisFb.noFcs / 3
        ReDim .trisFb.fc(1 To .trisFb.noFcs)
        Get coldHODno, , .trisFb.fc
        
        '  Output in order to check points in drawing program.
        'trisFileNo = FreeFile
        'Open "tris" & noCold & ".obj" For Append As #trisFileNo
        'Print #trisFileNo, ""
        'Print #trisFileNo, "g triangles"
        'For j = 1 To .trisFb.noVtcs
        '    Print #trisFileNo, "v " & .trisFb.vtx(j).x & " " & .trisFb.vtx(j).y & " " & .trisFb.vtx(j).z
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noVtcs & " vertices"
        'For j = 1 To .trisFb.noFcs
        '    Print #trisFileNo, "f  " & (.trisFb.fc(j).vtx(1) + 1) & " " & (.trisFb.fc(j).vtx(2) + 1) & " " & (.trisFb.fc(j).vtx(3) + 1)
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noFcs & " faces"
        'Print #trisFileNo, ""
        'Close #trisFileNo
        ' End of output section.
        
    End With
    
End Sub

Public Sub GetVarStr(openHODno As Integer, varStr As String, strLength As Long)

On Error GoTo eh

    varStr = Space(strLength)    ' Formatting the string before reading the name.
    Get openHODno, , varStr
    Exit Sub
    
eh:
    MsgBox "Unqualified string at " & Seek(openHODno)
    
End Sub

'***************************************************************************************************

Public Function genHODreport(HODname As String) As String

Dim repHeader As String
Dim repFooter As String
Dim i As Integer
Dim j As Integer
Dim k As Integer
Dim m As Integer
Dim tmpint As Integer
Dim tmpStr As String
Dim tmpBol As Boolean
Dim tmpLong As Long
Dim largestTag As Long
Dim GLOWfaceCounter(4) As Long
Dim MULTfaceCounter(7) As Long
Dim SSUBfacecounter() As Long
Dim LOD0meshCounter() As Integer
Dim LOD1meshCounter() As Integer

    repHeader = "{\rtf1\ansi\ansicpg1252\deff0\deflang1033{\fonttbl{\f0\fswiss\fcharset0 Arial;}}" _
                & Chr(13) & Chr(10) & "\viewkind4\uc1\pard\f0\fs20 \par" & Chr(13) & Chr(10) _
                & "Report on " & HODname & " HOD file: "
    tmpLong = ReverseLong(myHOD.versFh.lF)
    If tmpLong <> 8 Then
        repHeader = repHeader & "\par" & Chr(13) & Chr(10) & "Unusual HOD file length ! (Long, reversed): " _
                            & tmpLong & "\par" & Chr(13) & Chr(10)
    End If
    If myHOD.versFb.vNo <> 512 Then
        repHeader = repHeader & "\par" & Chr(13) & Chr(10) & "Unusual HOD version ! (Long): " _
                        & myHOD.versFb.vNo & "\par" & Chr(13) & Chr(10)
    End If
    
    ' HIER reporting:
    repHeader = repHeader & myHOD.dtrmFb.hierFb.noHobjects & " hierarchy objects\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
    If WARNpresence Then repHeader = repHeader & "HOD contains warning:  '" & myHOD.infoFb.warnFb.nWrn & "'.\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
    If ERRRpresence Then repHeader = repHeader & "HOD contains error message:  '" & myHOD.infoFb.errrFb.nErr & "'.\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10)
    repHeader = repHeader & "Absent chunks: "

    ' STATs reporting:
    tmpLong = UBound(myHOD.hvmdFb.statCh)
    tmpint = tmpLong + 1
    If Not STATpresence Then genHODreport = "\par" & Chr(13) & Chr(10) & "HOD without STAT headers !!!\par" & Chr(13) & Chr(10)
    genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpint & " STATs"
    tmpint = 0
    largestTag = 0
    tmpBol = False
    For i = 0 To tmpLong
        With myHOD.hvmdFb.statCh(i).statFb
            tmpint = tmpint + .noShaders
            For j = 1 To .noShaders
                If .shader(j).texTag > largestTag Then
                    largestTag = .shader(j).texTag
                End If
            Next j
            If InStr(.nTgrp, "lambert") Then tmpBol = True
        End With
    Next i
    genHODreport = genHODreport & " with a total of " & tmpint & " shaders "
    If tmpBol Then
        genHODreport = genHODreport & "(+ a 'lambert' shader). "
    Else
        genHODreport = genHODreport & "(no 'lambert' shader present). "
    End If
    
    ' LMIPs reporting:
    If LMIPpresence Then
        tmpLong = UBound(myHOD.hvmdFb.lmipCh)
        If largestTag <> tmpLong Then
            genHODreport = genHODreport & "The highest shader tag (" & largestTag _
                            & ", first tag = 0) DOES NOT CORRESPOND ( ! ) to the number of textures ( " & (tmpLong + 1) & " ).\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "Number of textures: " & (tmpLong + 1) & "\par" & Chr(13) & Chr(10)
                                                                            ' Plus one, because the LMIPs array starts at zero.
        End If
        tmpBol = False
        For i = 0 To tmpLong
                If myHOD.hvmdFb.lmipCh(i).lmipFb.lmipBh.tID = "8888" Then tmpBol = True    ' -> should be overall project setting?
        Next i
        If tmpBol Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & "The LMIP chunks contain Targa images.\par" & Chr(13) & Chr(10)
        End If
    Else
        genHODreport = genHODreport & "\par" & Chr(13) & Chr(10)
        repHeader = repHeader & "LMIP; "
    End If
    
    ' MULTs reporting:
    tmpLong = UBound(myHOD.hvmdFb.multCh)
    If tmpLong = 1 Then
        genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " MULT chunk with:\par" & Chr(13) & Chr(10) _
                                                        & "=-------------------------=\par" & Chr(13) & Chr(10)
    Else
        genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " MULTs with:\par" & Chr(13) & Chr(10) _
                                                        & "=-----------------=\par" & Chr(13) & Chr(10)
    End If
    
    For i = 1 To tmpLong
        
    
    Next i
    
    For i = 1 To tmpLong
        For j = 1 To myHOD.hvmdFb.multCh(i).multFb.noBMSHs
            With myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb
                For k = 1 To .noMshs
                    MULTfaceCounter(.LOD) = MULTfaceCounter(.LOD) + .shipmesh(k).noFcs
                Next k
            End With
        Next j
    Next i
    For i = 0 To 4
        If MULTfaceCounter(i) <> 0 Then
            genHODreport = genHODreport & "LOD" & i & ": " & MULTfaceCounter(i) & " faces\par" & Chr(13) & Chr(10)
        End If
    Next i
            
            ' LOD0 mesh number reporter per hierarchy object (+ shader indices):
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & "\tab " & "Number of LOD0 meshes per object (and their shader index):" & "\par" & Chr(13) & Chr(10)
            ReDim LOD0meshCounter(1 To tmpLong)
            For i = 1 To tmpLong
                For j = 1 To myHOD.hvmdFb.multCh(i).multFb.noBMSHs
                    If myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.LOD = 0 Then
                        LOD0meshCounter(i) = LOD0meshCounter(i) + myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                        For k = 1 To myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                            genHODreport = genHODreport & "\tab " & "\tab " & "MULT( " & i & " ), BMSH( " & j & " ), submesh( " & k & " ) has texTag " _
                                                        & myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).texTag & "\par" & Chr(13) & Chr(10)
                        Next k
                    End If
                Next j
                genHODreport = genHODreport & "\tab Total of " & LOD0meshCounter(i) & " meshes for object " & myHOD.hvmdFb.multCh(i).multFb.nO & "\par" & Chr(13) & Chr(10)
            Next i
            
            ' LOD1 mesh number reporter per hierarchy object (+ shader indices):
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & "\tab " & "Number of LOD1 meshes per object (and their shader index):" & "\par" & Chr(13) & Chr(10)
            ReDim LOD1meshCounter(1 To tmpLong)
            For i = 1 To tmpLong
                For j = 1 To myHOD.hvmdFb.multCh(i).multFb.noBMSHs
                    If myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.LOD = 1 Then
                        LOD1meshCounter(i) = LOD1meshCounter(i) + myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                        For k = 1 To myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.noMshs
                            genHODreport = genHODreport & "\tab " & "\tab " & "MULT( " & i & " ), BMSH( " & j & " ), submesh( " & k & " ) has texTag " _
                                                        & myHOD.hvmdFb.multCh(i).multFb.bmshCh(j).bmshFb.shipmesh(k).texTag & "\par" & Chr(13) & Chr(10)
                        Next k
                    End If
                Next j
                genHODreport = genHODreport & "\tab Total of " & LOD1meshCounter(i) & " meshes for object " & myHOD.hvmdFb.multCh(i).multFb.nO & "\par" & Chr(13) & Chr(10)
            Next i
    
    ' GOBGs reporting:
    If GOBGpresence Then
        tmpLong = UBound(myHOD.hvmdFb.gobgCh)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " GOBG chunk with "
        Else
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " GOBGs with "
        End If
        For i = 1 To tmpLong
            With myHOD.hvmdFb.gobgCh(i).gobgFb.bmshFb
                For j = 1 To .noMshs
                    MULTfaceCounter(5) = MULTfaceCounter(5) + .shipmesh(j).noFcs
                Next j
            End With
        Next i
        genHODreport = genHODreport & MULTfaceCounter(5) & " faces in total.\par" & Chr(13) & Chr(10)
    Else
        repHeader = repHeader & "GOBG; "
    End If
    
    ' ETSHs reporting:
    With myHOD.dtrmFb
        If ETSHpresence Then
            tmpLong = UBound(.etshFb)
            If tmpLong = 1 Then
                genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " ETSH chunk" & "\par" & Chr(13) & Chr(10) _
                                            & "=--------------=\par" & Chr(13) & Chr(10)
            Else
                genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " ETSHs" & "\par" & Chr(13) & Chr(10) _
                                            & "=---------=\par" & Chr(13) & Chr(10)
            End If
            For i = 1 To tmpLong
                tmpint = UBound(.etshFb(i).tSet)
                If tmpint = 1 Then
                    genHODreport = genHODreport & "\tab " & i & "... with " & .etshFb(i).tSet(1).noConnPnts & " shape points.\par" & Chr(13) & Chr(10)
                Else
                    genHODreport = genHODreport & "\tab " & i & "... with " & tmpint & " trail sets:\par" & Chr(13) & Chr(10)
                    For j = 1 To tmpint
                        genHODreport = genHODreport & "\tab " & "\tab " & j & "... with " & .etshFb(i).tSet(j).noConnPnts & " shape points.\par" & Chr(13) & Chr(10)
                    Next j
                End If
            Next i
        Else
            repHeader = repHeader & "ETSH; "
        End If
    End With
    
    ' GLOWs reporting:
    If GLOWpresence Then
        tmpLong = UBound(myHOD.dtrmFb.glowCh)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(10) & Chr(13) & tmpLong & " GLOW chunk with:\par" & Chr(13) & Chr(10) _
                                        & "=-----------------=\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(10) & Chr(13) & tmpLong & " GLOWs with:\par" & Chr(13) & Chr(10) _
                                        & "=-----------------=\par" & Chr(13) & Chr(10)
        End If
        For i = 1 To tmpLong
            With myHOD.dtrmFb.glowCh(i).glowFb
                For j = 1 To .bmshFb.noMshs
                    GLOWfaceCounter(.LOD) = GLOWfaceCounter(.LOD) + .bmshFb.shipmesh(j).noFcs
                Next j
            End With
        Next i
        For i = 0 To 4
            If GLOWfaceCounter(i) <> 0 Then
                genHODreport = genHODreport & "Total faces for LOD" & i & ": " & GLOWfaceCounter(i) & "\par" & Chr(13) & Chr(10)
            End If
        Next i
    Else
        repHeader = repHeader & "GLOW; "
    End If
    
    ' BURNs reporting:
    If BURNpresence Then
        tmpLong = UBound(myHOD.dtrmFb.burnFb)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(10) & Chr(13) & tmpLong & " BURN chunk\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(10) & Chr(13) & tmpLong & " BURNs\par" & Chr(13) & Chr(10)
        End If
    Else
        repHeader = repHeader & "BURN; "
    End If
    
    ' MRKRs reporting:
    If MRKRpresence Then
        tmpLong = UBound(myHOD.dtrmFb.mrkrCh)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " MRKR chunk\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " MRKRs\par" & Chr(13) & Chr(10)
        End If
        If MRKRseprDiff <> "" Then genHODreport = genHODreport & MRKRseprDiff & "\par" & Chr(13) & Chr(10)
    Else
        repHeader = repHeader & "MRKR; "
    End If
        
    ' NAVLs reporting:
    With myHOD.dtrmFb.navlFb
        If NAVLpresence Then
            tmpLong = UBound(.navl)
            If tmpLong = 1 Then
                genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " Navlight\par" & Chr(13) & Chr(10) _
                                            & "=-------------=\par" & Chr(13) & Chr(10)
            Else
                genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " Navlights\par" & Chr(13) & Chr(10) _
                                            & "=-------------=\par" & Chr(13) & Chr(10)
            End If
            For i = 1 To tmpLong
                genHODreport = genHODreport & .navl(i).nHo & "\tab " & .navl(i).n(1) _
                                            & "\tab " & .navl(i).n(2) & "\tab " & .navl(i).n(3) & "\tab " & .navl(i).n(4) _
                                            & "\tab " & .navl(i).o(1) & "\tab " & .navl(i).o(2) & "\tab " & .navl(i).o(3) _
                                            & "\tab " & .navl(i).o(4) & "\tab " & .navl(i).o(5) & "\par" & Chr(13) & Chr(10)
            Next i
        Else
            repHeader = repHeader & "NAVL; "
        End If
    End With
    
    ' DOCKs reporting:
    If DOCKpresence Then
        tmpLong = myHOD.dtrmFb.dockFb.noPaths
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " DOCK path\par" & Chr(13) & Chr(10) _
                                        & "=------------------=\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " DOCK paths\par" & Chr(13) & Chr(10) _
                                        & "=------------------=\par" & Chr(13) & Chr(10)
        End If
        For i = 1 To tmpLong
            With myHOD.dtrmFb.dockFb.path(i)
                genHODreport = genHODreport & .nP & "\tab " & .flag1 & "\tab " & .flag2 & "\tab " _
                                            & ReverseLong(.flag3) & "\tab " & .flag4 & "\tab " & .noPpoints & " pathpoints"
                If .noPars <> 10 Then genHODreport = genHODreport & " ( with " & .noPars & " parameters INSTEAD OF TEN ), "
                If .nAPs <> "" Then genHODreport = genHODreport & ", with | " & .nAPs & " | as alternative paths, "
                genHODreport = genHODreport & " for ship family: " & .nSfs & "\par" & Chr(13) & Chr(10)
                If .unknown8 <> 6 Then
                    genHODreport = genHODreport & "\tab " & " the ' unknown8 ' parameter is NOT EQUAL TO SIX ( " & .unknown8 & " );\par" & Chr(13) & Chr(10)
                End If
            End With
        Next i
    Else
        repHeader = repHeader & "DOCK; "
    End If
    
    ' ANIMs reporting:
    If ANIMpresence Then
        tmpLong = UBound(myHOD.dtrmFb.animCh)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " ANIM chunk" & "\par" & Chr(13) & Chr(10) _
                                                            & "=--------------=\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " ANIMs" & "\par" & Chr(13) & Chr(10) _
                                                            & "=--------------=\par" & Chr(13) & Chr(10)
        End If
        For i = 1 To tmpLong
            With myHOD.dtrmFb.animCh(i).animFb
                genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & .noKeyfs & " keyframes for " & .nPar & ":\par" & Chr(13) & Chr(10)
                For j = 1 To .noKeyfs
                    genHODreport = genHODreport & CSng(.keyframe(j).kTime) & "\par" & Chr(13) & Chr(10) _
                            & "\tab " & "\tab " & CSng(.keyframe(j).kPos.x) & "\tab " & CSng(.keyframe(j).kPos.y) & "\tab " & CSng(.keyframe(j).kPos.z) & "\par" & Chr(13) & Chr(10) _
                            & "\tab " & "\tab " & CSng(.keyframe(j).kRot.x) & "\tab " & CSng(.keyframe(j).kRot.y) & "\tab " & CSng(.keyframe(j).kRot.z) & "\par" & Chr(13) & Chr(10)
                Next j
            End With
        Next i
    Else
        repHeader = repHeader & "ANIM; "
    End If
        
    ' COLDs reporting:
    If COLDpresence Then
        tmpLong = UBound(myHOD.dtrmFb.coldCh)
        If tmpLong = 1 Then
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " COLD chunk, with TRIS vertices and faces:\par" & Chr(13) & Chr(10) _
                                & "=-------------------------------------------------------------=\par" & Chr(13) & Chr(10)
        Else
            genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " COLDs, with TRIS vertices and faces:\par" & Chr(13) & Chr(10) _
                                & "=--------------------------------------------------------=\par" & Chr(13) & Chr(10)
        End If
        For i = 1 To tmpLong        ' Usually only one COLD chunk (if at all) so a 'With' statement is not really useful here.
            genHODreport = genHODreport & myHOD.dtrmFb.coldCh(i).coldFb.trisFb.noVtcs & "\tab " & myHOD.dtrmFb.coldCh(i).coldFb.trisFb.noFcs _
                            & "\tab " & myHOD.dtrmFb.coldCh(i).coldFb.nO & "\par" & Chr(13) & Chr(10)
        Next i
    Else
        repHeader = repHeader & "COLD; "
    End If

    ' BSRMs reporting:
    With myHOD.hvmdFb
        tmpLong = UBound(myHOD.dtrmFb.bsrmCh)
        If GOBGpresence Then
            tmpBol = (tmpLong = (UBound(.multCh) + UBound(.gobgCh)))
        Else
            tmpBol = (tmpLong = UBound(.multCh))
        End If
    End With
    If tmpBol Then
        genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & "# BSRMs = # MULTs + # GOBGs (OK)," & "\tab " _
                                    & "(flags are used to indicate shader priority for objects).\par" & Chr(13) & Chr(10) _
                                    & "=------------------------------------------------------------------------------------------------------------------------------------------------------------------=\par" & Chr(13) & Chr(10)
    Else
        genHODreport = genHODreport & "\par" & Chr(13) & Chr(10) & tmpLong & " BSRMs DOES NOT CORRESPOND TO # MULTs + # GOBGs !!!\par" & Chr(13) & Chr(10)
    End If
    For i = 1 To tmpLong
        With myHOD.dtrmFb.bsrmCh(i).bsrmFb
            ' TRISs reporting:
            genHODreport = genHODreport & "TRIS faces : " & .trisFb.noFcs & ";\tab "
            ' VNRMs reporting:
            genHODreport = genHODreport & "VNRM 'faces' : " & .vnrmFb.noFcs & ";\tab "
            ' SSUBs reporting:
            largestTag = 0
            tmpint = 0
            For j = 1 To UBound(.ssubFb.fcLst)
                For k = 1 To UBound(.ssubFb.fcLst(j).faceIdx)
                    If .ssubFb.fcLst(j).faceIdx(k) > largestTag Then largestTag = .ssubFb.fcLst(j).faceIdx(k)
                Next k
            Next j
            If ((largestTag + 1) = .trisFb.noFcs) And ((largestTag + 1) = .vnrmFb.noFcs) Then
                genHODreport = genHODreport & "amount of SSUB face lists : " & "\tab " & UBound(.ssubFb.fcLst) & " ;\tab " & " flags: " _
                                & "\tab " & .ssubFb.flag(1) & "\tab " & .ssubFb.flag(2) & "\tab " & .ssubFb.flag(3) & "\tab "
            Else
                genHODreport = genHODreport & "amount of SSUB face lists : " & UBound(.ssubFb.fcLst) & " ;\tab " & " largest tag " _
                                            & largestTag & " NOT EQUAL to VNRM or TRIS faces!!!); flags : " & "\tab " & .ssubFb.flag(1) & "\tab " _
                                            & "\tab " & .ssubFb.flag(2) & "\tab " & .ssubFb.flag(3)
            End If
                                
            ' DESCs reporting:
            If .descFb.nonGoblin Then
                genHODreport = genHODreport & " -> " & .descFb.nO & ";\par" & Chr(13) & Chr(10)
                MULTfaceCounter(6) = MULTfaceCounter(6) + (largestTag + 1)
            Else
                genHODreport = genHODreport & " -> " & .descFb.nO & " = goblin mesh ( flag = " & .descFb.nonGoblin & " );\par" & Chr(13) & Chr(10)
                MULTfaceCounter(7) = MULTfaceCounter(7) + (largestTag + 1)
            End If
                
            '******** Below only necessary to generate more SSUB output.
            '                              ============================
            If (.descFb.nO = myHOD.dtrmFb.hierFb.hObj(1).nO) And .descFb.nonGoblin Then
                If (MsgBox("Generate all SSUB meshes for object ' " & myHOD.dtrmFb.hierFb.hObj(1).nO & " '?", vbYesNo, "SSUB mesh generator") = 6) Then genOBJfromFaces i
            End If
                
        End With
            
    Next i
        
    ' Special faces cross checks:
    If MULTfaceCounter(0) <> MULTfaceCounter(6) Then repFooter = "\par" & Chr(13) & Chr(10) & "\par" & Chr(13) & Chr(10) _
                                                                    & "**** The sum of all of the highest face indices of all non-goblin SSUB chunks" _
                                                                    & " DOES NOT CORRESPOND to the sum of all LOD0 faces in all (MULT) BMSH chunks ****"
    If MULTfaceCounter(5) <> MULTfaceCounter(7) Then repFooter = repFooter & "\par" & Chr(13) & Chr(10) _
                                                                    & "******** The sum of all of the highest face indices of all GOBLIN SSUB chunks" _
                                                                    & " DOES NOT CORRESPOND to the sum of all faces in all (GOBG) BMSH chunks ********"
    
    genHODreport = repHeader & "\par" & Chr(13) & Chr(10) & genHODreport & repFooter & "\par" & Chr(13) & Chr(10) & "}" & "\par" & Chr(13) & Chr(10)
    
End Function

Private Function genOBJfromFaces(noBsrm)

' Algorithm used to reconstruct & verify the
' meshes defined by the SSUB face-index lists.
' (by Pesmontis).

Dim objVtcs As String
Dim objFacs As String
Dim i As Integer
Dim j As Integer
Dim tmpint As Integer
Dim tmpTri As triangle
Dim partMesh As Integer

    With myHOD.dtrmFb.bsrmCh(noBsrm).bsrmFb
    
        For i = 1 To UBound(.ssubFb.fcLst)
        
            tmpint = .ssubFb.fcLst(i).noFcIdcs
            
            For j = 1 To tmpint
            
                tmpTri = .trisFb.fc(.ssubFb.fcLst(i).faceIdx(j) + 1)
                ' Plus one, because in the SSUB faces lists, the face indices start at zero,
                '  while the .trisFb.fc array starts at one.
                
                objVtcs = objVtcs & "v " & .trisFb.vtx(tmpTri.vtx(1) + 1).x & " " _
                                          & .trisFb.vtx(tmpTri.vtx(1) + 1).y & " " _
                                           & .trisFb.vtx(tmpTri.vtx(1) + 1).z & vbCrLf
                                 
                objVtcs = objVtcs & "v " & .trisFb.vtx(tmpTri.vtx(2) + 1).x & " " _
                                          & .trisFb.vtx(tmpTri.vtx(2) + 1).y & " " _
                                           & .trisFb.vtx(tmpTri.vtx(2) + 1).z & vbCrLf
                                           
                objVtcs = objVtcs & "v " & .trisFb.vtx(tmpTri.vtx(3) + 1).x & " " _
                                          & .trisFb.vtx(tmpTri.vtx(3) + 1).y & " " _
                                           & .trisFb.vtx(tmpTri.vtx(3) + 1).z & vbCrLf
                ' Plus one, because in the TRIS faces list, the vertex indices start at zero,
                '  while the .trisFb.vtx array starts at one.
                
                objFacs = objFacs & "f  " & (1 + (j - 1) * 3) & " " & (2 + (j - 1) * 3) & " " & (3 + (j - 1) * 3) & " " & vbCrLf
                
            Next j
            
            ' Output to OBJ format.
            partMesh = FreeFile
            If i < 10 Then
                Open .descFb.nO & "_00" & i & ".obj" For Append As #partMesh
            ElseIf i < 100 Then
                Open .descFb.nO & "_0" & i & ".obj" For Append As #partMesh
            Else
                Open .descFb.nO & "_" & i & ".obj" For Append As #partMesh
            End If
            Print #partMesh, vbCrLf & "g " & .descFb.nO & vbCrLf & objVtcs & "# " & (tmpint * 3) & " vertices" & vbCrLf & objFacs & "# " & tmpint & " faces" & vbCrLf
            Close #partMesh
            
            objVtcs = ""
            objFacs = ""
            
        Next i
        
    End With

End Function

Public Function openHW2shipHOD() As String

Dim dirAndFileName As String

    With frmConfigure.CommonDialog1
        ' Set CancelError is True
        .CancelError = False
        ' Set flags
        .flags = cdlOFNHideReadOnly
        ' Set filters
        .Filter = "Homeworld2 (ship type) HOD files (*.hod)|*.hod"
        .InitDir = App.path
        .ShowOpen
        If (.fileName <> "") And (InStr(.fileName, ".hod") <> 0) Then
            dirAndFileName = .fileName
            openHW2shipHOD = Mid$(.fileName, 1, ((InStr(.fileName, ".hod")) - 1))
            openHW2shipHOD = Mid$(openHW2shipHOD, ((InStrRev(openHW2shipHOD, "\")) + 1), Len(openHW2shipHOD))
            clearAllarrays
            importHW2shipHOD (dirAndFileName)
        Else
            MsgBox "Please select an Homeworld2 ship HOD file."
        End If
        
    End With

End Function

Public Function clearAllarrays()

    With myHOD.hvmdFb
        Erase .statCh
        Erase .lmipCh
        Erase .multCh
        Erase .gobgCh
    End With
    
    With myHOD.dtrmFb
        Erase .hierFb.hObj
        Erase .etshFb
        Erase .glowCh
        Erase .burnFb
        Erase .navlFb.navl
        Erase .dockFb.path
        Erase .mrkrCh
        Erase .animCh
        Erase .coldCh
        Erase .bsrmCh
    End With

End Function

Private Sub ReadBSRM(openHODno As Integer, noBsrm As Integer, lFb As Long)

Dim j As Integer
Dim k As Integer
Dim l As Integer
Dim trisFileNo As Integer
Dim ssubFileNo As Integer
Dim tmpLong As Long

    ReDim Preserve myHOD.dtrmFb.bsrmCh(1 To noBsrm)

    With myHOD.dtrmFb.bsrmCh(noBsrm).bsrmFh
        .tF = "FORM"
        .lF = lFb
    End With
    
    With myHOD.dtrmFb.bsrmCh(noBsrm).bsrmFb
        .tID = "BSRM"
        Get openHODno, , .descFh
        Get openHODno, , .descFb.tID
        Get openHODno, , .descFb.altLf
        Get openHODno, , .descFb.lN
        GetVarStr openHODno, .descFb.nM, .descFb.lN
        Get openHODno, , .descFb.lO
        GetVarStr openHODno, .descFb.nO, .descFb.lO
        Get openHODno, , .descFb.nonGoblin
        Get openHODno, , .bboxFh
        Get openHODno, , .bboxFb
        Get openHODno, , .bsphFh
        Get openHODno, , .bsphFb
        Get openHODno, , .trisFh
        Get openHODno, , .trisFb.tID
        Get openHODno, , .trisFb.lF
        Get openHODno, , .trisFb.noVtcs
        ReDim .trisFb.vtx(1 To .trisFb.noVtcs)
        Get openHODno, , .trisFb.vtx
        Get openHODno, , .trisFb.noFcs
        .trisFb.noFcs = .trisFb.noFcs / 3
        ReDim .trisFb.fc(1 To .trisFb.noFcs)
        Get openHODno, , .trisFb.fc
        Get openHODno, , .vnrmFh
        Get openHODno, , .vnrmFb.tID
        Get openHODno, , .vnrmFb.altLf
        Get openHODno, , .vnrmFb.noNrmls
        ReDim .vnrmFb.nrml(1 To .vnrmFb.noNrmls)
        Get openHODno, , .vnrmFb.nrml
        Get openHODno, , .vnrmFb.noFcs
        .vnrmFb.noFcs = .vnrmFb.noFcs / 3
        ReDim .vnrmFb.fc(1 To .vnrmFb.noFcs)
        Get openHODno, , .vnrmFb.fc
        Get openHODno, , .ssubFh
        Get openHODno, , .ssubFb.tID
        Get openHODno, , .ssubFb.altLf
'***
        Get openHODno, , .ssubFb.flag
        
        .ssubFh.lF = ((ReverseLong(.ssubFh.lF) - 20) / 2)   ' -> thus indicating how many integers are in the SSUB faces list.
        
        j = .ssubFh.lF   ' -> safety counter, indicating how many integers are still to be read.
        k = 1   '                                        =================
        ReDim .ssubFb.fcLst(1 To 1)
        
        While j > 0
            Get openHODno, , tmpLong
            j = j - 2
            If tmpLong > 0 Then
                ReDim Preserve .ssubFb.fcLst(1 To k)
                .ssubFb.fcLst(k).noFcIdcs = tmpLong
                j = j - tmpLong
                ReDim .ssubFb.fcLst(k).faceIdx(1 To tmpLong)
                Get openHODno, , .ssubFb.fcLst(k).faceIdx
                k = k + 1
            End If
        Wend
'***

        ' Output in order to validate DESC & SSUB parameters.
        'trisFileNo = FreeFile
        'Open "bmshtris" & noBsrm & ".obj" For Append As trisFileNo
        'Print #trisFileNo, ""
        'Print #trisFileNo, "g " & .descFb.nM
        'For j = 1 To .trisFb.noVtcs
        '    Print #trisFileNo, "v " & .trisFb.vtx(j).x & " " & .trisFb.vtx(j).y & " " & .trisFb.vtx(j).z
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noVtcs & "vertices"
        'For j = 1 To .trisFb.noFcs
        '    Print #trisFileNo, "f  " & (.trisFb.fc(j).vtx(1) + 1) & " " & (.trisFb.fc(j).vtx(2) + 1) & " " & (.trisFb.fc(j).vtx(3) + 1)
        'Next j
        'Print #trisFileNo, "# " & .trisFb.noFcs & " faces"
        'Print #trisFileNo, ""
        'Close #trisFileNo
        'ssubFileNo = FreeFile
        'Open "ssubCheck" & noBsrm & ".txt" For Append As #ssubFileNo
        'Print #ssubFileNo, "Output for " & noBsrm & "th BSRM - SSUB chunk."
        'Print #ssubFileNo, ""
        'For j = 1 To UBound(.ssubFb.fcLst)
        '    If .ssubFb.fcLst(j).sepr <> 0 Then
        '        Print #ssubFileNo, "Separator for " & j & "th face list = " & .ssubFb.fcLst(j).sepr
        '    End If
        '    Print #ssubFileNo, "Face list " & j & " has " & UBound(.ssubFb.fcLst(j).faceIdx) & " face indices."
        'Next j
        'Print #ssubFileNo, ""
        'Close #ssubFileNo
        ' End of output section.
        
    End With
                
End Sub

Private Sub ReadBURN(openHODno As Integer, noBurn As Integer)

Dim j As Integer
Dim burnFileNo As Integer

    ReDim Preserve myHOD.dtrmFb.burnFb(1 To noBurn)

    With myHOD.dtrmFb.burnFb(noBurn)
    
        .tID = "BURN"
        Get openHODno, , .lF
        Get openHODno, , .lN
        GetVarStr openHODno, .nB, .lN
        Get openHODno, , .lHo
        GetVarStr openHODno, .nHo, .lHo
        Get openHODno, , .noConnPnts
        Get openHODno, , .unknown4          ' This parameter is ALWAYS equal to one.
        ReDim .cPnt(1 To .noConnPnts)
        Get openHODno, , .cPnt
        
        ' Output in order to check points in drawing program.
        'burnFileNo = FreeFile
        'Open "burn" & noBurn & "Cpoints.obj" For Append As #burnFileNo
        'Print #burnFileNo, ""
        'Print #burnFileNo, "g " & .nB
        'For j = 1 To .noConnPnts
        '    Print #burnFileNo, "v " & .cPnt(j).x & " " & .cPnt(j).y & " " & .cPnt(j).z
        'Next j
        'Print #burnFileNo, "# " & .noConnPnts & " vertices"
        'Print #burnFileNo, ""
        'Close #burnFileNo
        ' End of output section.
        
    End With

End Sub

Private Sub ReadBNDV(openHODno As Integer)
    
    With myHOD.dtrmFb.bndvFb
        .tID = "BNVD"
        Get openHODno, , .lF
        Get openHODno, , .lN
        GetVarStr openHODno, .nAll, .lN
        Get openHODno, , .lO
        GetVarStr openHODno, .nO, .lO
        Get openHODno, , .noGenPts          ' This parameter is ALWAYS equal to eight.
        Get openHODno, , .leftLowerCorner
        Get openHODno, , .sepr1             ' This parameter-array ALWAYS contains only zero's.
        Get openHODno, , .rightUpperCorner
        Get openHODno, , .sepr2             ' This parameter-array ALWAYS contains only zero's.
    End With
    
End Sub

Private Sub ReadANIM(openHODno As Integer, noAnim As Integer, lFb As Long)

    ReDim Preserve myHOD.dtrmFb.animCh(1 To noAnim)

    With myHOD.dtrmFb.animCh(noAnim)
        .animFh.tF = "NRML"
        .animFh.lF = lFb
        .animFb.tID = "ANIM"
        Get openHODno, , .animFb.noAnimPars    ' This parameter is always equal to one (there's only ONE HOD with this chunk type!).
        Get openHODno, , .animFb.lNpar
        GetVarStr openHODno, .animFb.nPar, .animFb.lNpar
        Get openHODno, , .animFb.noKeyfs
        ReDim .animFb.keyframe(1 To .animFb.noKeyfs)
        Get openHODno, , .animFb.keyframe
        Get openHODno, , .animFb.sepr
    End With

End Sub

Public Function exportDDS(fileName As String, lmip As lmipBody) As String

' Write out the DDS file:
' a standard DDS header will have
' DDS File constants:
Const DDS_DWMAGIC As String = "DDS "
Const DDS_DWSIZE As Long = 124
Const DDS_DWFLAGS As Long = 659463
Const DDS_DWDEPTH As Long = 0
Const DDS_PIXELFORMAT_DWSIZE As Long = 32
Const DDS_PIXELFORMAT_DWFLAGS As Long = 4
Const DDS_PIXELFORMAT_DWRGBBITCOUNT As Long = 0
Const DDS_PIXELFORMAT_DWRBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWGBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWBBITMASK As Long = 0
Const DDS_PIXELFORMAT_DWRGBALPHABITMASK As Long = 0
Const DDS_DDSCAPS_DWCAPS1 As Long = 4198408
Const DDS_DDSCAPS_DWCAPS2 As Long = 0
Const DDS_DDSCAPS_DWRESERVED As Long = 0
Const DDS_DWRESERVED As Long = 0

Dim DDSfileOK As Boolean
Dim i As Integer
Dim j As Integer
Dim k As Integer
Dim myDDS As Integer

    With lmip
      myDDS = FreeFile
      exportDDS = fileName
      Open (exportDDS) For Binary As myDDS
      Put myDDS, , DDS_DWMAGIC
      Put myDDS, , DDS_DWSIZE
      Put myDDS, , DDS_DWFLAGS
      Put myDDS, , .lmipBh.Hi
      Put myDDS, , .lmipBh.Wi
      Put myDDS, , .lmipBh.Hi * .lmipBh.Wi
      Put myDDS, , DDS_DWDEPTH
      Put myDDS, , .lmipBh.noMIPls
      For k = 1 To 11
          Put myDDS, , DDS_DWRESERVED
      Next k
      Put myDDS, , DDS_PIXELFORMAT_DWSIZE
      Put myDDS, , DDS_PIXELFORMAT_DWFLAGS
      Put myDDS, , .lmipBh.tID
      Put myDDS, , DDS_PIXELFORMAT_DWRGBBITCOUNT
      Put myDDS, , DDS_PIXELFORMAT_DWRBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWGBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWBBITMASK
      Put myDDS, , DDS_PIXELFORMAT_DWRGBALPHABITMASK
      Put myDDS, , DDS_DDSCAPS_DWCAPS1
      Put myDDS, , DDS_DDSCAPS_DWCAPS2
      Put myDDS, , DDS_DDSCAPS_DWRESERVED
      Put myDDS, , DDS_DDSCAPS_DWRESERVED
      Put myDDS, , DDS_DWRESERVED
      Put myDDS, , .lmipBb()
      Close myDDS
    End With
    
End Function
