Attribute VB_Name = "modTempGetPeo"
Option Explicit

Dim myCore As clsCore
Dim myObjHost As hwseCore.IHostInterface

Private Type hwColourType
    r As Byte
    g As Byte
    b As Byte
    a As Byte
End Type

Private Type geoHeader
  identifier As String * 8
  version As Long
  name As Long
  fileSize As Long
  localSize As Long
  nPublicMaterials As Long
  nLocalMaterials As Long
  oPublicMaterials As Long
  oLocalMaterials As Long
  nPolygonObjects As Long
  Reserved As String * 24
End Type

' 32 bytes per material
Private Type tagMaterial
  nameOffset As Long
  ambient As hwColourType
  diffuse As hwColourType
  specular As hwColourType
  kAlpha As Single
  oTexture As Long
  flags As Integer
  nFullAmbient As Integer
  bTexturesRegistered As Integer
  textureNameSave As Integer
End Type

Private Type polygonObject
  nameOffset As Long
  flags As Long
  nVertices As Long
  nFaceNormals As Long
  nVertexNormals As Long
  nPolygons As Long
  oVertexList As Long
  oFaceNormalList As Long
  oVertNormalList As Long ' Cat only
  oPolygonList As Long
  oMother As Long
  oDaughter As Long
  oSister As Long
  matrix As D3DMATRIX
  localmatrix(16) As Single
  lmatrix As Single
  actualName As String ' Import only
End Type

Private Type tagVertex
  x As Single
  y As Single
  z As Single
  iVertexNormal As Long
End Type

Private Type tagNormal
  x As Single
  y As Single
  z As Single
  pad As String * 4
End Type

' 40 bytes long?
' 00 00 00 00 4D 00 57 00 31 00 00 00 01 00 22 3F 7D C7 CF 3D
' 22 DB 18 3F 10 9D 41 3E 01 00 22 3F 00 00 80 3E 00 08 00 78

Private Type tagPolygon
  iFaceNormal As Long       ' 00 00 00 00
  v0 As Integer              ' 4D 00
  v1 As Integer              ' 57 00
  v2 As Integer              ' 31 00
  iMaterial As Integer      ' 00 00
  s0 As Single              ' 00 01 22 3F
  t0 As Single              ' 7D C7 CF 3D
  s1 As Single              ' 22 D8 18 3F
  t1 As Single              ' 10 9D 41 3E
  s2 As Single              ' 01 00 00 22
  t2 As Single              ' 3F 00 00 80
  flags As Integer          ' 3E 00
  Reserved As Integer    ' 08 00
End Type

Private Type nodeObject
  Key As String
  Text As String
  Mother As Long
  Daughter As Long
  Sister As Long
  Added As Boolean
  localmatrix(16) As Single
End Type

Private ship As clsHwShip

Public Function importPEO(Filename As String, objHost As hwseCore.IHostInterface) As clsHwShip
'  errModule = "modPeo"
'  errFunction = "importPEO"
'  errParms = fileName

  Set myObjHost = objHost

'  On Error GoTo standardError
  
  'Debug.Print "-- In importPEO"
  
  'On Error GoTo ErrHandler

  ' Move this to the Plugin Init code!
  Set myCore = New clsCore
  Set ship = New clsHwShip
  
  myCore.errModule = "modPeo"
  myCore.errFunction = "importPeo"
  myCore.errParms = Filename
  
  Dim header As geoHeader
  Dim polygon As polygonObject
  Dim data As Byte
  Dim ObjectName As String
  Dim objNo As Integer
  Dim t As Integer
  Dim tmpLong As Long
  Dim tmpSharedOffsets() As Long
  Dim hwc As Boolean
  Dim filePos As Long
  
  myObjHost.DisplayText "Opening " & Filename & "..."
  
  ' Setup paths
  ship.Files.texturePath = Mid$(Filename, 1, InStrRev(Filename, "\"))
  myObjHost.DisplayText "Texture path: " & ship.Files.texturePath
  
  Open Filename For Binary As #1
  
  Get #1, , header.identifier
  Get #1, , header.version
  Get #1, , header.name
  Get #1, , header.fileSize
  Get #1, , header.localSize
  Get #1, , header.nPublicMaterials
  Get #1, , header.nLocalMaterials
  Get #1, , header.oPublicMaterials
  Get #1, , header.oLocalMaterials
  Get #1, , header.nPolygonObjects
  Get #1, , header.Reserved
  
  Select Case header.version
  Case 1026:
    ' Homeworld original .peo
    myObjHost.DisplayText vbCrLf & "-> HOMEWORLD peo file at hand,"
    ship.version = header.version
    'frmHWSE.StatusBar1.Panels(2).Text = "HW"
  Case 1280:
    ' Homeworld:Cataclysm
    myObjHost.DisplayText vbCrLf & "-> CATACLYSM peo file at hand,"
    ship.version = header.version
    'frmHWSE.StatusBar1.Panels(2).Text = "HW:C"
  End Select
  
  ship.numObjects = header.nPolygonObjects
  ship.PEO.numPMaterials = header.nPublicMaterials
  ship.PEO.numLMaterials = header.nLocalMaterials
  
  ' Create the shared offsets array
  ReDim tmpSharedOffsets(ship.numObjects - 1)

  myObjHost.DisplayText "     ( file size " & header.fileSize & " bytes, 'local size' is " & header.localSize & ","
  If ship.numObjects = 1 Then
    myObjHost.DisplayText "        containing ONE object only,"
  Else
    myObjHost.DisplayText "         containing " & ship.numObjects & " objects,"
  End If
  myObjHost.DisplayText "          " & header.nPublicMaterials & " public materials and " & header.nLocalMaterials & " local )."

  If header.nPublicMaterials = 0 Then
    ' It's a 3dex peo
    getMaterials header.nLocalMaterials, header.oLocalMaterials
  Else
    ' It's a "proper" Relic peo
    getMaterials header.nPublicMaterials, header.oPublicMaterials
  End If

  'polyNo = 0
  'vertexNo = 0
  'normalNo = 0

'  frmHWSE.prgLoadTextures.Max = header.nPolygonObjects
'  frmHWSE.prgLoadTextures.Value = 0
'
'  If hwsePrefs.objectSharing Then
'    objHost.DisplayText vbCrLf & "-> NOTE: object sharing is switched on" & vbCrLf
'  Else
'    objHost.DisplayText vbCrLf & "-> NOTE: object sharing is switched off" & vbCrLf
'  End If
'
  For objNo = 1 To header.nPolygonObjects

    Get #1, , polygon.nameOffset
    Get #1, , polygon.flags
    Get #1, , polygon.nVertices
    Get #1, , polygon.nFaceNormals
    Get #1, , polygon.nVertexNormals
    Get #1, , polygon.nPolygons
    Get #1, , polygon.oVertexList

    If ship.version = 1280 Then
      Get #1, , polygon.oVertNormalList
      Get #1, , polygon.oFaceNormalList
      Get #1, , polygon.oPolygonList
      hwc = True
    Else
      Get #1, , polygon.oFaceNormalList
      Get #1, , polygon.oPolygonList
      hwc = False
    End If

    ' Get object name from list
    filePos = Seek(1)
    Seek #1, polygon.nameOffset + 1
    ObjectName = ""
    Do
      Get #1, , data
      If data > 0 Then
        ObjectName = ObjectName & Chr$(data)
      End If
    Loop Until data = 0
    Seek #1, filePos

    objHost.DisplayText "Getting object " & ObjectName

'    frmHWSE.lblStatus = "Getting object " & ObjectName
'    frmHWSE.lblStatus.Refresh
'
    'redim the object list to the current object numbah.
    ship.PEO.Add objNo
    
'    ReDim Preserve ship.PEO.object(objNo)
    ship.PEO.object(objNo).objName = ObjectName 'set name of object

    ship.PEO.object(objNo).flags = polygon.flags
    ship.PEO.object(objNo).madObjNo = -1
    ship.PEO.object(objNo).sharedObjNo = -1

    objHost.DisplayText ship.PEO.object(objNo).objName & " #" & objNo & " loading"

'    ' Check the shared offsets list to see if these have
'    ' appeared before (ie this object shares it's details
'    ' with another)
'    If objNo > 0 Then
'      If hwsePrefs.objectSharing = True Then
'        For t = 0 To (objNo - 1)
'          If tmpSharedOffsets(t) = polygon.oPolygonList Then
'            ship.PEO.object(objNo).sharedObjNo = t
'            objHost.DisplayText "-> ** Shared Object with " & ship.PEO.object(t).objName & "**"
'            Exit For
'          End If
'          If ship.PEO.object(t).stats.nPolygons = polygon.nPolygons And ship.PEO.object(t).stats.nVertices = polygon.nVertices And ship.PEO.object(t).stats.nFaceNormals = polygon.nFaceNormals And ship.PEO.object(t).stats.nVertexNormals = polygon.nVertexNormals Then
'          ' Now there's reason to believe that this sub-object is a candidate for 'object sharing',
'          ' BUT it needs to be checked whether it really SHOULD be shared, because for instance
'          ' objects that are really a MIRRORED copies CANNOT be shared (!). Another example of unwanted
'          ' object sharing is when two completely different objects have the same amount of poly's and
'          ' vertices by accident. Therefor shared object detection MUST have some user input.
'          ' In fact, 'shared object detection' could be extended with an object NAME comparison,
'          ' but that would also require user acknowledgement.
'          ' (PJB, march 6, 2003).
'            msgMirror = vbCrLf & vbCrLf & "(Objects that are in fact MIRRORED copies of each other should NOT be shared!)."
'            o = MsgBox("HWSE thinks that " & ship.PEO.object(objNo).objName & " can be made a 'shared copy' of " & ship.PEO.object(t).objName & " ," & vbCrLf & "do you wish to share " & ship.PEO.object(objNo).objName & " as a clone of " & ship.PEO.object(t).objName & " ?" & msgMirror, vbQuestion + vbYesNo, "HWSE shared object detection")
'            If o = 6 Then
'              ship.PEO.object(objNo).sharedObjNo = t
'              objHost.DisplayText "-> ** Detected Shared Object with " & ship.PEO.object(t).objName & "**"
'              Exit For
'            End If
'          End If
'        Next t
'      End If
'    End If
'
'    ' Add this object's offset into the shared
'    ' offset list
'    tmpSharedOffsets(objNo) = polygon.oPolygonList
'
    ' Add object statistics
    ship.PEO.object(objNo).stats.nPolygons = polygon.nPolygons
    ship.PEO.object(objNo).stats.nVertices = polygon.nVertices
    ship.PEO.object(objNo).stats.nFaceNormals = polygon.nFaceNormals
    ship.PEO.object(objNo).stats.nVertexNormals = polygon.nVertexNormals

    ' Add overall statistics
    'shipStats.nPolygons = shipStats.nPolygons + polygon.nPolygons
    'shipStats.nVertices = shipStats.nVertices + polygon.nVertices
'
'    pLog "-- Stats:", 1
'    pLog "   nPolygons: " & polygon.nPolygons, 1
'    pLog "      offset: " & polygon.oPolygonList, 1
'    pLog "   nVertices: " & polygon.nVertices, 1
'    pLog "      offset: " & polygon.oVertexList, 1
'    pLog "   nFaceNormals: " & polygon.nFaceNormals, 1
'    pLog "         offset: " & polygon.oFaceNormalList, 1
'    pLog "   nVertNormals: " & polygon.nVertexNormals, 1
'    pLog "         offset: " & polygon.oVertNormalList, 1
'    pLog "  actual offset: " & polygon.oFaceNormalList + (polygon.nFaceNormals * 16), 1
'
'    pLog "-> Flags: " & polygon.flags, 2
'    tmpLong = polygon.flags
'    pLog "          " & DecToBin(tmpLong), 3
'
    objHost.DisplayText "-> Vertices: " & polygon.nVertices & " @ " & polygon.oVertexList
    getVertices polygon.oVertexList + 1, polygon.nVertices, objNo

    objHost.DisplayText "-> FaceNormals: " & polygon.nFaceNormals & " @ " & polygon.oFaceNormalList
    getFaceNormals polygon.oFaceNormalList + 1, polygon.nFaceNormals, objNo
'
    If hwc = True Then
      objHost.DisplayText "-> cVertexNormals: " & polygon.nVertexNormals & " @ " & polygon.oVertNormalList
'      'getVertNormals polygon.oVertNormalList + 1, polygon.nVertexNormals, objNo
'      getVertNormals polygon.oFaceNormalList + 1, polygon.nVertexNormals, objNo
    Else
      objHost.DisplayText "-> VertexNormals: " & polygon.nVertexNormals
    End If
    getVertNormals polygon.oFaceNormalList + 1, polygon.nVertexNormals, objNo

    objHost.DisplayText "-> Polygons: " & polygon.nPolygons & " @ " & polygon.oPolygonList
    getFaces polygon.oPolygonList + 1, polygon.nPolygons, objNo

    Get #1, , polygon.oMother
    Get #1, , polygon.oDaughter
    Get #1, , polygon.oSister
    Get #1, , polygon.matrix

    With ship.PEO.object(objNo)
      .Mother.offset = polygon.oMother
      .Mother.Value = convertOffset(polygon.oMother, hwc)
      .Daughter.offset = polygon.oDaughter
      .Daughter.Value = convertOffset(polygon.oDaughter, hwc)
      .Sister.offset = polygon.oSister
      .Sister.Value = convertOffset(polygon.oSister, hwc)
      
      objHost.DisplayText "MDS: " & .Mother.Value & " / " & .Daughter.Value & " / " & .Sister.Value

    End With
    
    objHost.DisplayText "Calling transposeMatrix..."
    transposeMatrix polygon.matrix, objNo
    
    'ship.PEO.object(objNo).matrix = transposeMatrix(polygon.matrix)
    
    'objHost.DisplayText "Position: " & ship.PEO.object(objNo).matrix.rc14 & " " & ship.PEO.object(objNo).matrix.rc24 & " " & ship.PEO.object(objNo).matrix.rc34
    
    'frmHWSE.prgLoadTextures.Value = objNo + 1
  Next objNo
'  --- END OF COMMENT ---

EndOfPeo:
  
  Close #1
  
  ' Assign back by ref
  Set importPEO = ship
  
  'frmHWSE.lblStatus2 = ""
  'lightShip
  
  Exit Function

'errHandler:
  'Debug.Print "getVertexList - " & Err.Number & " (" & Err.Description & ")"
  
  'Exit Sub

standardError:
  myCore.ShowError vbModal
End Function

Function getMaterials(noOfMaterials As Long, vOffset As Long) As Boolean
  Dim oldFilePos As Long, filePos As Long

  'errModule = "modPeo"
  'errFunction = "getMaterials"
  'errParms = noOfMaterials & "|" & vOffset
  On Error GoTo standardError
  
  'Debug.Print noOfMaterials & " materials loading..."
  oldFilePos = Seek(1)

  'ReDim matNames(noOfMaterials - 1)
  'ReDim ship.materials(noOfMaterials - 1)
  
  ' Setup our type
  Dim material As tagMaterial
  Dim data As Byte
  Dim materialname As String
  Dim pWidth As Long
  Dim pHeight As Long
  Dim t As Integer
  Dim foundTex As Boolean
  Dim m As Integer
  Dim tmpColour As hwColourType
  Dim layerType As Integer
  Dim maxTextures As Integer
  Dim matOffset As Long
  
  ' Seek to the position of the material list
  Seek #1, vOffset + 1
  
  ship.numMaterials = noOfMaterials
  If ship.numMaterials = 1 Then
    myObjHost.DisplayText vbCrLf & "Loading one material:"
  Else
    myObjHost.DisplayText vbCrLf & "Loading " & ship.numMaterials & " materials:"
  End If
  maxTextures = -1
  
  'frmHWSE.lblStatus = "Getting materials"
  'frmHWSE.lblStatus.Refresh
  'frmHWSE.prgLoadTextures.Max = noOfMaterials
  'frmHWSE.prgLoadTextures.Value = 0
  
  Dim myMaterial As clsHwMaterialType
  
  Dim j As Integer
  For j = 1 To noOfMaterials
    ship.materials.Add j - 1
  
    'Set myMaterial = New clsHwMaterialType
    
    matOffset = Seek(1)
    Get #1, , material.nameOffset
    Get #1, , material.ambient
    Get #1, , material.diffuse
    Get #1, , material.specular
    Get #1, , material.kAlpha
    Get #1, , material.oTexture
    Get #1, , material.flags
    Get #1, , material.nFullAmbient
    Get #1, , material.bTexturesRegistered
    Get #1, , material.textureNameSave

    'ship.materials(j - 1).flags = material.flags
    Set ship.materials(j - 1).ambient = checkMaterial(material.ambient)
    Set ship.materials(j - 1).diffuse = checkMaterial(material.diffuse)
    Set ship.materials(j - 1).specular = checkMaterial(material.specular)
    ship.materials(j - 1).kAlpha = material.kAlpha
    ship.materials(j - 1).nFullAmbient = material.nFullAmbient
    ship.materials(j - 1).offset = matOffset

' Have to figure out a way of doing this config stuff in the plugin
'    ' Added PJB, march 9, 2003
'    If frmOptions.chkSwitchClrs = 1 Then
'      tmpColour.r = ship.materials(j - 1).ambient.r
'      ship.materials(j - 1).ambient.r = ship.materials(j - 1).ambient.b
'      ship.materials(j - 1).ambient.b = tmpColour.r
'      tmpColour.r = ship.materials(j - 1).diffuse.r
'      ship.materials(j - 1).diffuse.r = ship.materials(j - 1).diffuse.b
'      ship.materials(j - 1).diffuse.b = tmpColour.r
'      tmpColour.r = ship.materials(j - 1).specular.r
'      ship.materials(j - 1).specular.r = ship.materials(j - 1).specular.b
'      ship.materials(j - 1).specular.b = tmpColour.r
'    End If

'    Set ship.materials(j - 1).material = frmHWSE.RMCanvas1.dX.Direct3DRMCreate.CreateMaterial(0)
     'Set ship.materials(j - 1) = objHost.CreateMaterial(myMaterial)
     'objHost.CreateMaterial myMaterial

'    With ship.materials(j - 1).material
'      .SetAmbient material.ambient.r, material.ambient.g, material.ambient.b
'      '.SetPower material.diffuse.r
'      .SetSpecular material.specular.r, material.specular.g, material.specular.b
'    End With
'
    ' Convert material flags to actual boolean values
    Dim flagBits As String, tmpLong As Long, fb As Integer
    tmpLong = material.flags
    flagBits = DecToBin(tmpLong)
    'myObjHost.DisplayText j - 1 & ": " & tmpLong & ":" & flagBits
    For fb = 1 To 17
      If Mid$(flagBits, fb, 1) = "1" Then
        Select Case fb
        Case 16:
          ' MDF_Smoothing 2
          ship.materials(j - 1).flags.smoothed = True
        Case 15:
          ' MDF_AllowFlatShading 4
          ship.materials(j - 1).flags.flatShaded = True
        Case 14:
          ' MDF_2Sided 8
          ship.materials(j - 1).flags.twoSided = True
        Case 13:
          ' MDF_BaseColor 16
          ship.materials(j - 1).flags.baseColour = True
        Case 12:
          ' MDF_StripeColor 32
          ship.materials(j - 1).flags.stripeColour = True
        Case 11:
          ' MDF_SelfIlluminated
          ship.materials(j - 1).flags.selfIllum = True
        End Select
      End If
    Next fb

'    'Debug.Print "bTexReg: " & material.bTexturesRegistered
'
'    'm = j

    filePos = Seek(1)
    'myObjHost.DisplayText "Seeking to: " & material.oTexture + 1
    Seek #1, material.oTexture + 1
    materialname = ""
    Do
      Get #1, , data
      If data > 0 Then
        materialname = materialname & Chr$(data)
      End If
    Loop Until data = 0
    
    myObjHost.DisplayText "Material: " & materialname
    Seek #1, filePos
    
    Dim found As Boolean
    found = False

    Dim w As Integer
    Dim texNo As Integer
    If maxTextures > -1 Then
      For w = 0 To maxTextures
'        'Debug.Print "Checking " & ship.textures(w).texName & " against " & materialname
        If ship.textures(w).texName = materialname Then
          'Debug.Print "Found " & materialname
          found = True
          texNo = w
        End If
      Next w
    Else
      found = False
    End If

    If found = False Then
      maxTextures = maxTextures + 1
      ship.textures.Add maxTextures
'      ReDim Preserve ship.textures(maxTextures)

'      'Debug.Print "+++ Increasing textureHigh: " & maxTextures
      texNo = maxTextures
'
      ship.textures(maxTextures).texName = materialname
      'myObjHost.DisplayText "Texture " & maxTextures & ": " & ship.textures(maxTextures).texName
'      ' Turned off loading of LIFs for faster load time
'      ' we don't really need them yet
'      'LoadFile materialname & ".lif"
'      'ship.textures(maxTextures).lWidth = LiFHeader.Width
'      'ship.textures(maxTextures).lHeight = LiFHeader.Height
    End If
'
    ship.materials(j - 1).texture = texNo
'    pText "- ' " & materialname & " '      ( #" & j - 1 & " & texture #" & texNo & " )"
'    'pText "            " & material.nameOffset & " " & material.oTexture
'    frmHWSE.prgLoadTextures.Value = frmHWSE.prgLoadTextures.Value + 1
'    Seek #1, filePos
  Next j
  
  myObjHost.DisplayText "Total textures: " & ship.textures.Count
  ' Rename the texture names to include the extension
  ' you MUST do this in order for the main program
  ' to recognise the texture type
  For j = 0 To (ship.textures.Count - 1)
    ship.textures(j).texName = ship.textures(j).texName & ".lif"
  Next
  
'  frmHWSE.lblStatus = "Getting Textures"
'  frmHWSE.lblStatus.Refresh
'
'  Dim startTime As Long
'  startTime = frmHWSE.RMCanvas1.dX.TickCount
'
'  If UBound(ship.textures) >= 0 Then
'
'  frmHWSE.prgLoadTextures.Max = UBound(ship.textures) + 1
'  frmHWSE.prgLoadTextures.Value = 0
'
'  matPath = Mid$(frmHWSE.StatusBar1.Panels(1).Text, 1, InStrRev(frmHWSE.StatusBar1.Panels(1).Text, "\"))
'  For j = 0 To UBound(ship.textures)
'    materialname = ship.textures(j).texName
'    frmHWSE.lblStatus = "Getting Texture " & materialname & ".lif"
'    frmHWSE.lblStatus.Refresh
'
'    'Debug.Print "Checking " & materialname
'    ' First check for a previously made BMP file for
'    ' quicker loading
'    'If Dir(matPath & materialname & ".bmp") = "" Then
'      ' If no BMP file found then we have to draw to the
'      ' DD surface (slower)
'      If Dir(matPath & materialname & ".lif") <> "" Then
'        'Debug.Print "Loading material " & materialname
'        frmHWSE.lstTextures.ListItems.Add , , materialname
'
'        ' Delphy 28/01/03: Added for treeView
'        'frmHWSE.tvwTextures.Nodes.Add , tvwFirst, "txr" & j, materialname
'
'        If LoadFile(matPath & materialname & ".lif", True) = True Then
'          frmHWSE.lstTextures.ListItems(j + 1).SubItems(1) = myLifFile.lifHeader.Width
'          frmHWSE.lstTextures.ListItems(j + 1).SubItems(2) = myLifFile.lifHeader.Height
'          frmHWSE.lstTextures.ListItems(j + 1).SubItems(3) = myLifFile.lifHeader.flags
'
'          'drawLif materialname
'          If hwsePrefs.drawTeamColours = True Then
'            layerType = 7
'          Else
'            layerType = 1
'          End If
'
'          'If hwsePrefs.noDDlifs = True Then
'            frmLifPreview.StatusBar1.Panels(4) = layerType
'            frmLifPreview.drawLayer
'            SavePicture frmLifPreview.picLif1.Image, matPath & materialname & ".bmp"
'           frmHWSE.ctlThumbs.Add matPath & materialname & ".bmp", True
'           pLog "  -> lif texture " & j & " as " & matPath & materialname & ".bmp", 3
'            Set ship.textures(j).Main = RM.LoadTexture(matPath & materialname & ".bmp")
'            Kill matPath & materialname & ".bmp"
'          'Else
'          '  Set ship.textures(j).main = CreateUpdateableTexture(layerType)
'          'End If
'
'          ship.textures(j).loaded = True
'        End If
'      Else
'        frmHWSE.lstTextures.ListItems.Add , , materialname
'
'        ' Delphy 28/03/01: Added for texture treeview
'        'frmHWSE.tvwTextures.Nodes.Add , tvwFirst, "txr" & j, materialname
'
'        frmHWSE.ctlThumbs.Add matPath & materialname & ".lif", True
'        ship.textures(j).loaded = False
'      End If
'    'Else
'    '  frmHWSE.lstTextures.ListItems.Add , , materialname
'    '
'    '  LoadFile matPath & materialname & ".lif", False
'    '  frmHWSE.lstTextures.ListItems(j + 1).SubItems(1) = lifHeader.Width
'    '  frmHWSE.lstTextures.ListItems(j + 1).SubItems(2) = lifHeader.Height
'    '  frmHWSE.lstTextures.ListItems(j + 1).SubItems(3) = lifHeader.flags
'    '
'    '  pLog "  -> bmp texture " & j & " as " & matPath & materialname & ".bmp", 3
'      ' If we have a BMP then we can simply load this
'    '  Set ship.textures(j).main = RM.LoadTexture(matPath & materialname & ".bmp")
'    '  If hwsePrefs.cleanupBMPs = True Then
'    '    Kill matPath & materialname & ".bmp"
'    '  End If
'    '  ship.textures(j).loaded = True
'    '  frmHWSE.List1.AddItem materialname
'    'End If
'    frmHWSE.prgLoadTextures.Value = frmHWSE.prgLoadTextures.Value + 1
'  Next j
  
'  End If
  Seek #1, oldFilePos

  'frmHWSE.ctlThumbs.Caption = UBound(ship.textures) + 1 & " textures loaded"

  Exit Function

standardError:
  'frmError.Show vbModal
End Function


Private Function checkMaterial(mat As hwColourType) As clsHwColourType
  Set checkMaterial = New clsHwColourType
  
  With mat
    If .a > 0 Then
      ' This is a 3dex saved Lif so swap RGB
      checkMaterial.r = .b
      checkMaterial.g = .g
      checkMaterial.b = .r
    Else
      checkMaterial.r = .r
      checkMaterial.g = .g
      checkMaterial.b = .b
    End If
  End With

End Function

Private Sub getVertices(vertexOffset As Long, numOfVerts As Long, objNo As Integer)

  Dim tmpVertex As tagVertex
  
  ' Seek to the position of the Vertex List
  Dim oldFilePos As Long, i As Long
  oldFilePos = Seek(1)
  Seek #1, vertexOffset

  For i = 1 To numOfVerts
    Get #1, , tmpVertex.x
    Get #1, , tmpVertex.y
    Get #1, , tmpVertex.z
    Get #1, , tmpVertex.iVertexNormal
    With ship.PEO.object(objNo)
      .AddVertex i
      ' Flip z?
      .Vertex(i).x = tmpVertex.x
      .Vertex(i).y = tmpVertex.y
      .Vertex(i).z = tmpVertex.z
      .Vertex(i).normal = tmpVertex.iVertexNormal
    End With
  Next i
    
  Seek #1, oldFilePos

End Sub

Private Sub getFaceNormals(normalOffset As Long, numOfNormals As Long, objNo As Integer)
  ' Setup our type
  Dim normal As tagNormal
  Dim oldFilePos As Long, i As Long

  oldFilePos = Seek(1)
  Seek #1, normalOffset

  For i = 1 To numOfNormals
    Get #1, , normal.x
    Get #1, , normal.y
    Get #1, , normal.z
    Get #1, , normal.pad
    With ship.PEO.object(objNo)
      .AddFaceNormal i
      .FaceNormalList(i).x = normal.x
      .FaceNormalList(i).y = normal.y
      .FaceNormalList(i).z = normal.z
    End With
  Next i

  Seek #1, oldFilePos

End Sub

Private Sub getVertNormals(normalOffset As Long, numOfNormals As Long, objNo As Integer)
  ' Setup our type
  Dim normal As tagNormal
  Dim oldFilePos As Long, i As Long

  oldFilePos = Seek(1)
  Seek #1, normalOffset

  For i = 1 To ship.PEO.object(objNo).stats.nFaceNormals
    Get #1, , normal.x
    Get #1, , normal.y
    Get #1, , normal.z
    Get #1, , normal.pad
  Next i
  
  For i = 1 To numOfNormals
    Get #1, , normal.x
    Get #1, , normal.y
    Get #1, , normal.z
    Get #1, , normal.pad
    With ship.PEO.object(objNo)
      .AddVertexNormal i
      .VertexNormalList(i).x = normal.x
      .VertexNormalList(i).y = normal.y
      .VertexNormalList(i).z = normal.z
    End With
  Next i

  Seek #1, oldFilePos

End Sub

Private Sub getFaces(faceOffset As Long, numOfFaces As Long, objNo As Integer)

  ' Setup our type
  Dim polygonE As tagPolygon
  Dim oldFilePos As Long, i As Long
  Dim tmpLong As Long
  Dim tmpString As String
  Dim corruptPolys As Integer
  
  ' Seek to the polygon list
  oldFilePos = Seek(1)
  Seek #1, faceOffset
  
  'Debug.Print ship.PEO.object(j - 1).objName
  myObjHost.DisplayText "-> Loading Polygons: " & faceOffset
    
  corruptPolys = 0
  
  For i = 1 To numOfFaces
    Get #1, , polygonE.iFaceNormal
    Get #1, , polygonE.v0
    Get #1, , polygonE.v1
    Get #1, , polygonE.v2
    Get #1, , polygonE.iMaterial
    Get #1, , polygonE.s0
    Get #1, , polygonE.t0
    Get #1, , polygonE.s1
    Get #1, , polygonE.t1
    Get #1, , polygonE.s2
    Get #1, , polygonE.t2
    Get #1, , polygonE.flags
    Get #1, , polygonE.Reserved
    
    With ship.PEO.object(objNo)
      .AddFace i
      ' Note since face numbers run from 1 this has to be done
      ' Since we have to convert to left-handed, swap the polygon order
      .Faces(i).Vertex.v0 = polygonE.v0 + 1
      .Faces(i).Vertex.v1 = polygonE.v1 + 1
      .Faces(i).Vertex.v2 = polygonE.v2 + 1
      
      If (polygonE.v1 = polygonE.v0) Then
        myObjHost.DisplayText " - corrupted polygon found"
        corruptPolys = corruptPolys + 1
      ElseIf (polygonE.v2 = polygonE.v0) Or (polygonE.v2 = polygonE.v1) Then
        myObjHost.DisplayText "- corrupted polygon found"
        corruptPolys = corruptPolys + 1
      End If
      
      .Faces(i).material = polygonE.iMaterial
      .Faces(i).normal = polygonE.iFaceNormal
      .Faces(i).TexList.v0t = polygonE.s0
      .Faces(i).TexList.v0u = polygonE.t0
      .Faces(i).TexList.v1t = polygonE.s1
      .Faces(i).TexList.v1u = polygonE.t1
      .Faces(i).TexList.v2t = polygonE.s2
      .Faces(i).TexList.v2u = polygonE.t2
      .Faces(i).flags = polygonE.flags
      .Faces(i).Reserved = polygonE.Reserved
      'tmpLong = .flags
      'tmpString = DecToBin(tmpLong)
            
      'myObjHost.DisplayText "   " & Format(i - 1, "@@@@") & ": vert: " & Format(.Vertex(0), "@@@@") & "/" & Format(.Vertex(1), "@@@@") & "/" & Format(.Vertex(2), "@@@@") & " mat: " & Format(.material, "@@@") & " normal: " & Format(.normal, "@@@") & " flags: " & Format(.flags, "@@@@") & " " & Mid$(tmpString, 3, 4) & " (" & BinToDec(Mid$(tmpString, 3, 4)) & ") " & Mid$(tmpString, 7, 4) & " (" & BinToDec(Mid$(tmpString, 7, 4)) & ") " & Mid$(tmpString, 11, 4) & " (" & BinToDec(Mid$(tmpString, 11, 4)) & ")", 3
      
    End With
      
  Next i

  If corruptPolys <> 0 Then
    myObjHost.DisplayText "Polygon loader found " & corruptPolys & " corrupt polys in " & numOfFaces & " polys for object " & objNo & "."
  End If

  Seek #1, oldFilePos

End Sub

Public Function DecToBin(inputNumber As Long) As String

    Dim Bits As Integer
    Bits = 16 ' a decimal value from 0 to 65535
    
    Do While Bits >= 0
        If (inputNumber >= 2 ^ Bits) Then
            inputNumber = inputNumber - 2 ^ Bits
            DecToBin = DecToBin & "1"
        Else
            DecToBin = DecToBin & "0"
        End If
        Bits = Bits - 1
    Loop

End Function


Function convertOffset(offset As Long, isHwcShip As Boolean)
  
  ' SM_WARNING this may need changing after I've re-ordered the arrays to be from 0 to x-1
  ' Added support for Cataclysm, which has an object size of
  ' 116.
  If offset = 0 Then
    convertOffset = offset
  Else
    If isHwcShip Then
      convertOffset = ((offset - 68) / 116) + 1
    Else
      convertOffset = ((offset - 68) / 112) + 1
    End If
  End If

End Function

Public Sub transposeMatrix(iMatrix As D3DMATRIX, objNo As Integer)

  With ship.PEO.object(objNo).matrix
    If frmConfigure.chkTransposeMatrix.Value = 1 Then
      .rc11 = iMatrix.rc11
      .rc12 = iMatrix.rc21
      .rc13 = iMatrix.rc31
      .rc14 = iMatrix.rc41
      .rc21 = iMatrix.rc12
      .rc22 = iMatrix.rc22
      .rc23 = iMatrix.rc32
      .rc24 = iMatrix.rc42
      .rc31 = iMatrix.rc13
      .rc32 = iMatrix.rc23
      .rc33 = iMatrix.rc33
      .rc34 = iMatrix.rc43
      .rc41 = iMatrix.rc14
      .rc42 = iMatrix.rc24
      .rc43 = iMatrix.rc34
      .rc44 = iMatrix.rc44
    Else
      .rc11 = iMatrix.rc11
      .rc12 = iMatrix.rc12
      .rc13 = iMatrix.rc13
      .rc14 = iMatrix.rc14
      .rc21 = iMatrix.rc21
      .rc22 = iMatrix.rc22
      .rc23 = iMatrix.rc23
      .rc24 = iMatrix.rc24
      .rc31 = iMatrix.rc31
      .rc32 = iMatrix.rc32
      .rc33 = iMatrix.rc33
      .rc34 = iMatrix.rc34
      .rc41 = iMatrix.rc41
      .rc42 = iMatrix.rc42
      .rc43 = iMatrix.rc43
      .rc44 = iMatrix.rc44
    End If
  End With
End Sub

