Attribute VB_Name = "modMathFunctions"
Public Const pi = 3.14159265359

Public Function DecToBin(inputNumber As Long) As String

    Dim Bits As Integer
    Bits = 16 ' a decimal value from 0 to 65535
    
    Do While Bits >= 0
        If (inputNumber >= 2 ^ Bits) Then
            inputNumber = inputNumber - 2 ^ Bits
            DecToBin = DecToBin & "1"
        Else
            DecToBin = DecToBin & "0"
        End If
        Bits = Bits - 1
    Loop

End Function

Public Function DecToBinary(ByVal decnum As Long) As String
  
  Select Case decnum
    Case Is > 1
      DecToBinary = DecToBinary(decnum \ 2) & CStr(decnum And 1)
    Case Is > 0
      DecToBinary = "1"
    Case Else
      DecToBinary = "0"
  End Select
  
End Function

'  Converts a number in any base from 2 to 36  to a long.
'  Note that this doesn't verify if the string is a valid number in the given base.
Public Function Base2Long(s As String, ByVal nB As Integer) As Long
  errModule = "modMathFunctions"
  errFunction = "Base2Long"
  errParms = s & "|" & nB
  On Error GoTo standardError
   
   Dim s2 As String
   Dim i As Double
   Dim j As Double
   Dim x As Double
   Dim n As Boolean
   Dim s3 As String
   
   If Len(s) < 1 Then
      Base2Long = 0
      Exit Function
   End If
   
   s2 = UCase(s)
   
   If Left$(s2, 1) = "-" Then
      n = True
      s2 = Right$(s2, Len(s2) - 1)
   Else
      n = False
   End If
   
   j = 1
   x = 0
   
   For i = Len(s2) To 1 Step -1
      s3 = Mid$(s2, i, 1)
      Select Case s3
      Case "0" To "9":
         x = x + j * (Asc(s3) - 48)
      Case "A" To "Z":
         x = x + j * (Asc(s3) - 55)
      End Select
      
      j = j * nB
   Next i
   
   If n Then
      x = -x
   End If
   
   Base2Long = x
   
  Exit Function
standardError:
  frmError.Show vbModal
End Function

' Converts the number n to any base between 2 and 36.
Public Function Long2Base(ByVal n As Long, ByVal nB As Integer) As String
  errModule = "modMathFunctions"
  errFunction = "long2base"
  errParms = n & "|" & nB
  On Error GoTo standardError
  
  Dim s As String
  Dim nD As Integer
  Dim Negative As Boolean

  Negative = n < 0
  n = Abs(n)
  
  Do
    nD = n Mod nB
    If nD > 9 Then
       nD = nD + 7
    End If
    
    s = Chr$(48 + nD) & s
    n = n \ nB
  Loop Until n = 0
  
  If Negative Then
    s = "-" & s
  End If
  
  Long2Base = s
  
  Exit Function
standardError:
  frmError.Show vbModal
End Function

Function convHex(inputNumber As Variant)
  errModule = "modMathFunctions"
  errFunction = "convHex"
  errParms = inputNumber
  On Error GoTo standardError
  
  Dim hexString As String, temp As String, temp2 As String
  Dim i As Integer

  hexString = Hex(inputNumber)
  Do While (Len(hexString) Mod 2) = 1
    hexString = "0" & hexString
  Loop
  For i = Len(hexString) To 2 Step -2
    temp = temp & Mid$(hexString, i - 1, 2)
  Next i
  If Mid$(temp, 1, 2) = "00" Then
    temp = Mid$(temp, 3)
    For i = Len(temp) To 2 Step -2
      temp2 = temp2 & Mid$(temp, i - 1, 2)
    Next i
  Else
    temp2 = temp
  End If
  convHex = temp2
  
  Exit Function
standardError:
  frmError.Show vbModal
End Function

' Reverses the HEX equivalent of a number (useful for storing).
' The output is a number in big- or little-endian format (I forget which)).
Function ReverseHex(inputNumber As Variant)
  errModule = "modMathFunctions"
  errFunction = "ReverseHex"
  errParms = inputNumber
  On Error GoTo standardError
  
  Dim hexString As String, temp As String
  Dim i As Integer

  hexString = Hex(inputNumber)
  Do While (Len(hexString) Mod 2) = 1
    hexString = "0" & hexString
  Loop
  For i = Len(hexString) To 2 Step -2
    temp = temp & Mid$(hexString, i - 1, 2)
  Next i
  ReverseHex = temp
  
  Exit Function
standardError:
  frmError.Show vbModal
End Function

Public Function BinToDec(inputBinary As String) As Long
  
  Dim tmpVar As Long, tmpBits As Long
  Dim i As Integer
  tmpBits = 0
    
  For i = Len(inputBinary) To 1 Step -1
    If tmpBits = 0 Then tmpBits = 1 Else tmpBits = tmpBits * 2
    If Mid$(inputBinary, i, 1) = 1 Then tmpVar = tmpVar + tmpBits
  Next i

  BinToDec = tmpVar

End Function

' Convert from degrees to radians
Function DegToRad(degrees)

  DegToRad = degrees / 180 * pi

End Function

' Convert from radians to degrees
Function RadToDeg(radians) As Single

  RadToDeg = radians * 180 / pi
  
End Function

' Calculates a matrix based on Euler angles of rotation, following the rotation order z -> y -> x.
Function calcMatrix(calcEuler As D3DVECTOR) As D3DMATRIX
  
  Dim sinX As Double, sinY As Double, sinZ As Double
  Dim cosX As Double, cosY As Double, cosZ As Double
  
  With calcEuler
    sinX = Sin(.x): sinY = Sin(.Y): sinZ = Sin(.z)
    cosX = Cos(.x): cosY = Cos(.Y): cosZ = Cos(.z)
  End With
  
  With calcMatrix
    .rc11 = cosY * cosZ
    .rc12 = cosY * sinZ
    .rc13 = -sinY
    .rc21 = sinX * sinY * cosZ - cosX * sinZ
    .rc22 = sinX * sinY * sinZ + cosX * cosZ
    .rc23 = sinX * cosY
    .rc31 = cosX * sinY * cosZ + sinX * sinZ
    .rc32 = cosX * sinY * sinZ - sinX * cosZ
    .rc33 = cosX * cosY
    
    If Abs(.rc11) < 0.001 Then .rc11 = 0#
    If Abs(.rc12) < 0.001 Then .rc12 = 0#
    If Abs(.rc13) < 0.001 Then .rc13 = 0#
    If Abs(.rc21) < 0.001 Then .rc21 = 0#
    If Abs(.rc22) < 0.001 Then .rc22 = 0#
    If Abs(.rc23) < 0.001 Then .rc23 = 0#
    If Abs(.rc31) < 0.001 Then .rc31 = 0#
    If Abs(.rc32) < 0.001 Then .rc32 = 0#
    If Abs(.rc33) < 0.001 Then .rc33 = 0#
  End With
  
End Function

Public Function transposeMatrix(iMatrix As D3DMATRIX) As D3DMATRIX

  transposeMatrix.rc11 = iMatrix.rc11
  transposeMatrix.rc12 = iMatrix.rc21
  transposeMatrix.rc13 = iMatrix.rc31
  transposeMatrix.rc14 = iMatrix.rc41
  transposeMatrix.rc21 = iMatrix.rc12
  transposeMatrix.rc22 = iMatrix.rc22
  transposeMatrix.rc23 = iMatrix.rc32
  transposeMatrix.rc24 = iMatrix.rc42
  transposeMatrix.rc31 = iMatrix.rc13
  transposeMatrix.rc32 = iMatrix.rc23
  transposeMatrix.rc33 = iMatrix.rc33
  transposeMatrix.rc34 = iMatrix.rc43
  transposeMatrix.rc41 = iMatrix.rc14
  transposeMatrix.rc42 = iMatrix.rc24
  transposeMatrix.rc43 = iMatrix.rc34
  transposeMatrix.rc44 = iMatrix.rc44

End Function

Public Sub dumpMatrix(srcMatrix As D3DMATRIX)
  
  Open App.Path & "\testmatrix.log" For Output As #88
  With srcMatrix
    Print #88, .rc11, .rc12, .rc13, .rc14
    Print #88, .rc21, .rc22, .rc23, .rc24
    Print #88, .rc31, .rc32, .rc33, .rc34
    Print #88, .rc41, .rc42, .rc43, .rc44
  End With
  Close #88
  
End Sub

' In order to calculate arcsine (inverse sine) it is necessary to use the VB function atan() (Pesmontis).
Public Function asin(sineAngle As Double) As Double
  
  asin = Atn(sineAngle / Sqr(1 - sineAngle ^ 2))

End Function

' In order to calculate arccosine (inverse cosine) it is necessary to use the VB function atan() (Pesmontis).
Public Function acos(cosineAngle As Double) As Double

  acos = Atn(Sqr(1 - cosineAngle ^ 2) / cosineAngle)

End Function

