Attribute VB_Name = "modPlugins"
Option Explicit

Public nPlugins() As clsPluginType
'Public objHost As hwseCore.IHostInterface

Public Function LoadPlugins(pluginsDir As String, myobjHost As IHostInterface) As Boolean
    Dim strDlls() As String
    Dim iCounter As Integer, i As Integer, j As Integer
    Dim iPluginCount As Integer, iFailCount As Integer
    Dim strInterfaceName As String
    
    Dim objTypeInfo As TypeLibInfo

    'Initialize dynamic array of plugin information
    ReDim nPlugins(0)
    
    'Set up our interface which we'll pass to plugins to they can call back
    'Set objHost = New clsHostInterface
    
    'Enumerate DLL files in plugins directory
    If EnumFiles(pluginsDir & "*.dll", strDlls) = True Then
        For iCounter = 0 To UBound(strDlls)
            'Get typelib information from the DLL file
            Set objTypeInfo = TLI.TypeLibInfoFromFile(pluginsDir & strDlls(iCounter))
            objTypeInfo.Register
            
            For j = 1 To objTypeInfo.TypeInfos.Count
                For i = 1 To objTypeInfo.TypeInfos(j).Interfaces.Count
                    'This is where we'll get an error if something is wrong with the DLL itself
                    On Local Error Resume Next
                    strInterfaceName = objTypeInfo.TypeInfos(j).Interfaces(i).name
                    If Err Then
                        Debug.Print strInterfaceName & " Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        myobjHost.DisplayText strInterfaceName & " Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        strInterfaceName = ""
                        iFailCount = iFailCount + 1
                        Err.Clear
                    End If
                    On Error GoTo 0
                    
                    Debug.Print "interface name: " & strInterfaceName
                    'Test for required interfaces
                    If Mid$(strInterfaceName, 1, 5) = "_hwse" Then
                        'We've found what appears to be a compatible plugin
                        If LoadPlugin(pluginsDir & strDlls(iCounter), objTypeInfo.name & "." & objTypeInfo.TypeInfos(j).name) = False Then
                            myobjHost.DisplayText "Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                        Else
                            myobjHost.DisplayText "Loaded plugin: " & strDlls(iCounter) & " v" & nPlugins(UBound(nPlugins)).version
                            myobjHost.DisplayText "--> " & nPlugins(UBound(nPlugins)).PluginType
                            iPluginCount = iPluginCount + 1
                        End If
                    End If
                    'If strInterfaceName = "_IOperation2" Then
                    '    'We've found what appears to be a compatible plugin
                    '    If LoadPlugin2(App.Path & "\Plugins\" & strDlls(iCounter), objTypeInfo.Name & "." & objTypeInfo.TypeInfos(j).Name) = False Then
                    '        objHost.DisplayText "Failed to load plugin " & Chr$(34) & strDlls(iCounter) & Chr$(34)
                    '    Else
                    '        objHost.DisplayText "Loaded output plugin: " & strDlls(iCounter)
                    '        iPluginCount = iPluginCount + 1
                    '    End If
                    'End If
                Next i
            Next j
        Next
    Else
        LoadPlugins = False
        Exit Function
    End If
    
    'See if we loaded any plugins successfully
    If iPluginCount > 0 Then
        myobjHost.DisplayText CStr(iPluginCount) & " Plugin DLL(s) loaded successfully."
        LoadPlugins = True
    Else
        LoadPlugins = False
    End If
End Function

Private Function LoadPlugin(strDLLPath As String, strClassName As String) As Boolean

    Debug.Print "LoadPlugin: "
    Debug.Print "--> " & strDLLPath
    Debug.Print "--> " & strClassName

    Dim objPlugin As hwseCore.hwsePlugin
    Dim bError As Boolean

    On Local Error Resume Next

    'Try and create an instance of the plugin
    Set objPlugin = CreateObject(strClassName)
    Debug.Print "Error: " & Err

    'If we fail, try registering the DLL
    If Err Then
        'This is probably because the DLL isn't registered, so we try and register it
        If RegisterServer(0, strDLLPath, True) = True Then
            Err.Clear
            Set objPlugin = CreateObject(strClassName)
            If Err Then bError = True
        Else
            bError = True
        End If
    End If

    Debug.Print bError

    'bError indicates whether we've been able to create an instance of the plugin
    If bError = True Then
        LoadPlugin = False
    Else
        'NOTE: We don't actually store an instance of the plugin in our array.
        'This would probably be the best way to do it for this tiny sample
        'application, but in real life it is usually best to create and destroy
        'instances when we need them. We do, however keep a copy of the plugin
        'information.

        'Create a new entry in our plugin database for this plugin
        ReDim Preserve nPlugins(UBound(nPlugins) + 1)
        'nPlugins(UBound(nPlugins)).strClassName = strClassName
        nPlugins(UBound(nPlugins)) = objPlugin.PluginInfo
        nPlugins(UBound(nPlugins)).version = GetVersionInfo(strDLLPath)
        LoadPlugin = True
    End If

    Set objPlugin = Nothing
End Function

